// This code conforms with the UFC specification version 2018.1.0
// and was automatically generated by FFC version 2019.2.0.dev0.
//
// This code was generated with the option '-l dolfin' and
// contains DOLFIN-specific wrappers that depend on DOLFIN.
//
// This code was generated with the following parameters:
//

//  add_tabulate_tensor_timing:     False
//  convert_exceptions_to_warnings: False
//  cpp_optimize:                   True
//  cpp_optimize_flags:             '-O2'
//  epsilon:                        1e-14
//  error_control:                  False
//  external_include_dirs:          ''
//  external_includes:              ''
//  external_libraries:             ''
//  external_library_dirs:          ''
//  form_postfix:                   True
//  format:                         'dolfin'
//  generate_dummy_tabulate_tensor: False
//  max_signature_length:           0
//  optimize:                       True
//  precision:                      None
//  quadrature_degree:              None
//  quadrature_rule:                None
//  representation:                 'auto'
//  split:                          False

#ifndef __BIHARMONIC_H
#define __BIHARMONIC_H
#include <algorithm>
#include <cmath>
#include <iostream>
#include <stdexcept>
#include <ufc.h>

class biharmonic_finite_element_0: public ufc::finite_element
{
public:

  biharmonic_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Lagrange', triangle, 1)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 3;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 1;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  void evaluate_reference_basis(double * reference_values,
                                std::size_t num_points,
                                const double * X) const final override
  {
    static const double coefficients0[1][3] = { { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients1[1][3] = { { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients2[1][3] = { { 0.4714045207910316, 0.0, 0.3333333333333333 } };
    for (std::size_t k = 0; k < num_points * 3; ++k)
        reference_values[k] = 0.0;
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues1[3] = {};
        basisvalues1[0] = 1.0;
        const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues1[1] = tmp1_1;
        basisvalues1[2] = (0.5 + 1.5 * Y[1]) * basisvalues1[0];
        basisvalues1[0] *= std::sqrt(0.5);
        basisvalues1[2] *= std::sqrt(1.0);
        basisvalues1[1] *= std::sqrt(3.0);
        // Accumulate products of coefficients and basisvalues
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[3 * ip] += coefficients0[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[3 * ip + 1] += coefficients1[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[3 * ip + 2] += coefficients2[0][r] * basisvalues1[r];
    }
  }

  void evaluate_reference_basis_derivatives(double * reference_values,
                                            std::size_t order,
                                            std::size_t num_points,
                                            const double * X) const final override
  {
    if (order == 0)
    {
        evaluate_reference_basis(reference_values, num_points, X);
        return;
    }
    const std::size_t num_derivatives = std::pow(2, order);
    std::fill_n(reference_values, num_points * 3 * num_derivatives, 0.0);
    if (order > 1)
        return;
    // Tables of derivatives of the polynomial base (transpose).
    alignas(32) static const double dmats0[2][3][3] =
        { { { 0.0, 0.0, 0.0 },
            { 4.8989794855663495, 0.0, 0.0 },
            { 0.0, 0.0, 0.0 } },
          { { 0.0, 0.0, 0.0 },
            { 2.449489742783182, 0.0, 0.0 },
            { 4.242640687119285, 0.0, 0.0 } } };
    static const double coefficients0[1][3] = { { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients1[1][3] = { { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients2[1][3] = { { 0.4714045207910316, 0.0, 0.3333333333333333 } };
    const std::size_t reference_offset[3] = {};
    const std::size_t num_components[3] = { 1, 1, 1 };
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues1[3] = {};
        basisvalues1[0] = 1.0;
        const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues1[1] = tmp1_1;
        basisvalues1[2] = (0.5 + 1.5 * Y[1]) * basisvalues1[0];
        basisvalues1[0] *= std::sqrt(0.5);
        basisvalues1[2] *= std::sqrt(1.0);
        basisvalues1[1] *= std::sqrt(3.0);
        // Loop over all dofs
        for (std::size_t i = 0; i < 3; ++i)
        {
            double derivatives[2] = {};
            switch (i)
            {
            case 0:
                // Compute reference derivatives for dof 0.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients0[0][s] * aux[s];
                }
                break;
            case 1:
                // Compute reference derivatives for dof 1.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients1[0][s] * aux[s];
                }
                break;
            case 2:
                // Compute reference derivatives for dof 2.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients2[0][s] * aux[s];
                }
                break;
            }
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t c = 0; c < num_components[i]; ++c)
                    reference_values[3 * num_derivatives * ip + num_derivatives * i + r + (reference_offset[i] + c)] = derivatives[num_derivatives * c + r];
        }
    }
  }

  void transform_reference_basis_derivatives(double * values,
                                             std::size_t order,
                                             std::size_t num_points,
                                             const double * reference_values,
                                             const double * X,
                                             const double * J,
                                             const double * detJ,
                                             const double * K,
                                             int cell_orientation) const final override
  {
    const std::size_t num_derivatives = std::pow(2, order);
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    std::fill_n(values, num_points * 3 * num_derivatives, 0.0);
    const std::size_t reference_offsets[3] = {};
    const std::size_t physical_offsets[3] = {};
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        double transform[2][2];
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                transform[r][s] = 1.0;
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                for (std::size_t k = 0; k < order; ++k)
                    transform[r][s] *= K[2 * 2 * ip + 2 * combinations[order - 1][s][k] + combinations[order - 1][r][k]];
        for (std::size_t d = 0; d < 3; ++d)
        {
            for (std::size_t s = 0; s < num_derivatives; ++s)
            {
                for (std::size_t i = 0; i < 1; ++i)
                {
                    // Using affine transform to map values back to the physical element.
                    const double mapped_value = reference_values[3 * num_derivatives * ip + num_derivatives * d + s + reference_offsets[d]];
                    // Mapping derivatives back to the physical element
                    for (std::size_t r = 0; r < num_derivatives; ++r)
                        values[3 * num_derivatives * ip + num_derivatives * d + r + (physical_offsets[d] + i)] += transform[r][s] * mapped_value;
                }
            }
        }
    }
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    double X[2] = {};
    double J[4];
    double detJ;
    double K[4];
    if (cm)
    {
        cm->compute_reference_geometry(X, J, &detJ, K, 1, x, coordinate_dofs, cell_orientation);
    }
    else
    {
        compute_jacobian_triangle_2d(J, coordinate_dofs);
        compute_jacobian_inverse_triangle_2d(K, detJ, J);
        // Compute constants
        const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
        const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
        // Get coordinates and map to the reference (FIAT) element
        double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
        // Map to FFC reference coordinate
        for (std::size_t k = 0; k < 2; ++k)
            X[k] = (Y[k] + 1.0) / 2.0;
    }
    // Evaluate basis on reference element
    double ref_values[3];
    evaluate_reference_basis(ref_values, 1, X);
    // Push forward
    double physical_values[3];
    transform_reference_basis_derivatives(physical_values, 0, 1, ref_values, X, J, &detJ, K, cell_orientation);
    for (std::size_t k = 0; k < 1; ++k)
        values[k] = physical_values[i + k];
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation,
                          const ufc::coordinate_mapping * cm=nullptr
                          ) const final override
  {
    // Helper variable to hold value of a single dof.
    double dof_values = 0.0;
    // Loop dofs and call evaluate_basis
    for (std::size_t r = 0; r < 3; ++r)
    {
        evaluate_basis(r, &dof_values, x, coordinate_dofs, cell_orientation);
        values[r] = dof_values;
    }
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation,
                                  const ufc::coordinate_mapping * cm=nullptr
                                  ) const final override
  {
    std::size_t num_derivatives = std::pow(2, n);
    std::fill_n(values, num_derivatives, 0.0);
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
        return;
    }
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
        return;
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    // Compute Inverse Jacobian and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    // Get coordinates and map to the reference (FIAT) element
    double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    // Declare transformation matrix
    double transform[2][2] =
        { { 1.0, 1.0 },
          { 1.0, 1.0 } };
    // Construct transformation matrix
    for (std::size_t row = 0; row < num_derivatives; ++row)
        for (std::size_t col = 0; col < num_derivatives; ++col)
            for (std::size_t k = 0; k < n; ++k)
                transform[row][col] *= K[2 * combinations[n - 1][col][k] + combinations[n - 1][row][k]];
    switch (i)
    {
    case 0:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 1:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 2:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910316, 0.0, 0.3333333333333333 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    }
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation,
                                      const ufc::coordinate_mapping * cm=nullptr
                                      ) const final override
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
        return;
    }
    unsigned int num_derivatives = std::pow(2, n);
    // Set values equal to zero.
    std::fill_n(values, num_derivatives * 3, 0.0);
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
        return;
    // Helper variable to hold values of a single dof.
    double dof_values[2] = {};
    // Loop dofs and call evaluate_basis_derivatives.
    for (std::size_t r = 0; r < 3; ++r)
    {
        evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
        for (std::size_t s = 0; s < num_derivatives; ++s)
            values[num_derivatives * r + s] = dof_values[s];
    }
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
        {
            y[0] = coordinate_dofs[0];
            y[1] = coordinate_dofs[1];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 1:
        {
            y[0] = coordinate_dofs[2];
            y[1] = coordinate_dofs[3];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 2:
        {
            y[0] = coordinate_dofs[4];
            y[1] = coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    }
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c,
                             const ufc::coordinate_mapping * cm=nullptr
                             ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::coordinate_mapping * cm=nullptr
                                 ) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs,
                                const ufc::coordinate_mapping * cm=nullptr
                                ) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[2 + 1] = coordinate_dofs[3];
    dof_coordinates[2 * 2] = coordinate_dofs[4];
    dof_coordinates[2 * 2 + 1] = coordinate_dofs[5];
  }

  void tabulate_reference_dof_coordinates(double * reference_dof_coordinates) const final override
  {
    static const double dof_X[6] = { 0.0, 0.0, 1.0, 0.0, 0.0, 1.0 };
    std::copy_n(dof_X, 6, reference_dof_coordinates);
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_0();
  }

};


class biharmonic_finite_element_1: public ufc::finite_element
{
public:

  biharmonic_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "VectorElement(FiniteElement('Lagrange', triangle, 1), dim=2)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 6;
  }

  std::size_t value_rank() const final override
  {
    return 1;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    static const int return_values[1] = { 2 };
    if (i >= 1)
        return 1;
    return return_values[i];
  }

  std::size_t value_size() const final override
  {
    return 2;
  }

  std::size_t reference_value_rank() const final override
  {
    return 1;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    static const int return_values[1] = { 2 };
    if (i >= 1)
        return 1;
    return return_values[i];
  }

  std::size_t reference_value_size() const final override
  {
    return 2;
  }

  std::size_t degree() const final override
  {
    return 1;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  void evaluate_reference_basis(double * reference_values,
                                std::size_t num_points,
                                const double * X) const final override
  {
    static const double coefficients0[1][3] = { { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients1[1][3] = { { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients2[1][3] = { { 0.4714045207910316, 0.0, 0.3333333333333333 } };
    for (std::size_t k = 0; k < num_points * 6 * 2; ++k)
        reference_values[k] = 0.0;
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues1[3] = {};
        basisvalues1[0] = 1.0;
        const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues1[1] = tmp1_1;
        basisvalues1[2] = (0.5 + 1.5 * Y[1]) * basisvalues1[0];
        basisvalues1[0] *= std::sqrt(0.5);
        basisvalues1[2] *= std::sqrt(1.0);
        basisvalues1[1] *= std::sqrt(3.0);
        // Accumulate products of coefficients and basisvalues
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip] += coefficients0[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip + 2] += coefficients1[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip + 2 * 2] += coefficients2[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip + 2 * 3 + 1] += coefficients0[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip + 2 * 4 + 1] += coefficients1[0][r] * basisvalues1[r];
        for (std::size_t r = 0; r < 3; ++r)
            reference_values[6 * 2 * ip + 2 * 5 + 1] += coefficients2[0][r] * basisvalues1[r];
    }
  }

  void evaluate_reference_basis_derivatives(double * reference_values,
                                            std::size_t order,
                                            std::size_t num_points,
                                            const double * X) const final override
  {
    if (order == 0)
    {
        evaluate_reference_basis(reference_values, num_points, X);
        return;
    }
    const std::size_t num_derivatives = std::pow(2, order);
    std::fill_n(reference_values, num_points * 6 * num_derivatives * 2, 0.0);
    if (order > 1)
        return;
    // Tables of derivatives of the polynomial base (transpose).
    alignas(32) static const double dmats0[2][3][3] =
        { { { 0.0, 0.0, 0.0 },
            { 4.8989794855663495, 0.0, 0.0 },
            { 0.0, 0.0, 0.0 } },
          { { 0.0, 0.0, 0.0 },
            { 2.449489742783182, 0.0, 0.0 },
            { 4.242640687119285, 0.0, 0.0 } } };
    static const double coefficients0[1][3] = { { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients1[1][3] = { { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 } };
    static const double coefficients2[1][3] = { { 0.4714045207910316, 0.0, 0.3333333333333333 } };
    const std::size_t reference_offset[6] = { 0, 0, 0, 1, 1, 1 };
    const std::size_t num_components[6] = { 1, 1, 1, 1, 1, 1 };
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues1[3] = {};
        basisvalues1[0] = 1.0;
        const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues1[1] = tmp1_1;
        basisvalues1[2] = (0.5 + 1.5 * Y[1]) * basisvalues1[0];
        basisvalues1[0] *= std::sqrt(0.5);
        basisvalues1[2] *= std::sqrt(1.0);
        basisvalues1[1] *= std::sqrt(3.0);
        // Loop over all dofs
        for (std::size_t i = 0; i < 6; ++i)
        {
            double derivatives[2] = {};
            switch (i)
            {
            case 0:
                // Compute reference derivatives for dof 0.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients0[0][s] * aux[s];
                }
                break;
            case 1:
                // Compute reference derivatives for dof 1.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients1[0][s] * aux[s];
                }
                break;
            case 2:
                // Compute reference derivatives for dof 2.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients2[0][s] * aux[s];
                }
                break;
            case 3:
                // Compute reference derivatives for dof 3.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients0[0][s] * aux[s];
                }
                break;
            case 4:
                // Compute reference derivatives for dof 4.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients1[0][s] * aux[s];
                }
                break;
            case 5:
                // Compute reference derivatives for dof 5.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[3] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[3][3] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 9, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[3][3];
                        std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 9, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 3; ++s)
                        for (std::size_t t = 0; t < 3; ++t)
                            aux[s] += dmats[s][t] * basisvalues1[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 3; ++s)
                        derivatives[r] += coefficients2[0][s] * aux[s];
                }
                break;
            }
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t c = 0; c < num_components[i]; ++c)
                    reference_values[6 * (num_derivatives * 2) * ip + num_derivatives * 2 * i + 2 * r + (reference_offset[i] + c)] = derivatives[num_derivatives * c + r];
        }
    }
  }

  void transform_reference_basis_derivatives(double * values,
                                             std::size_t order,
                                             std::size_t num_points,
                                             const double * reference_values,
                                             const double * X,
                                             const double * J,
                                             const double * detJ,
                                             const double * K,
                                             int cell_orientation) const final override
  {
    const std::size_t num_derivatives = std::pow(2, order);
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    std::fill_n(values, num_points * 6 * num_derivatives * 2, 0.0);
    const std::size_t reference_offsets[6] = { 0, 0, 0, 1, 1, 1 };
    const std::size_t physical_offsets[6] = { 0, 0, 0, 1, 1, 1 };
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        double transform[2][2];
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                transform[r][s] = 1.0;
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                for (std::size_t k = 0; k < order; ++k)
                    transform[r][s] *= K[2 * 2 * ip + 2 * combinations[order - 1][s][k] + combinations[order - 1][r][k]];
        for (std::size_t d = 0; d < 6; ++d)
        {
            for (std::size_t s = 0; s < num_derivatives; ++s)
            {
                for (std::size_t i = 0; i < 1; ++i)
                {
                    // Using affine transform to map values back to the physical element.
                    const double mapped_value = reference_values[6 * (num_derivatives * 2) * ip + num_derivatives * 2 * d + 2 * s + reference_offsets[d]];
                    // Mapping derivatives back to the physical element
                    for (std::size_t r = 0; r < num_derivatives; ++r)
                        values[6 * (num_derivatives * 2) * ip + num_derivatives * 2 * d + 2 * r + (physical_offsets[d] + i)] += transform[r][s] * mapped_value;
                }
            }
        }
    }
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    double X[2] = {};
    double J[4];
    double detJ;
    double K[4];
    if (cm)
    {
        cm->compute_reference_geometry(X, J, &detJ, K, 1, x, coordinate_dofs, cell_orientation);
    }
    else
    {
        compute_jacobian_triangle_2d(J, coordinate_dofs);
        compute_jacobian_inverse_triangle_2d(K, detJ, J);
        // Compute constants
        const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
        const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
        // Get coordinates and map to the reference (FIAT) element
        double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
        // Map to FFC reference coordinate
        for (std::size_t k = 0; k < 2; ++k)
            X[k] = (Y[k] + 1.0) / 2.0;
    }
    // Evaluate basis on reference element
    double ref_values[12];
    evaluate_reference_basis(ref_values, 1, X);
    // Push forward
    double physical_values[12];
    transform_reference_basis_derivatives(physical_values, 0, 1, ref_values, X, J, &detJ, K, cell_orientation);
    for (std::size_t k = 0; k < 2; ++k)
        values[k] = physical_values[2 * i + k];
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation,
                          const ufc::coordinate_mapping * cm=nullptr
                          ) const final override
  {
    // Helper variable to hold values of a single dof.
    double dof_values[2] = {};
    // Loop dofs and call evaluate_basis
    for (std::size_t r = 0; r < 6; ++r)
    {
        evaluate_basis(r, dof_values, x, coordinate_dofs, cell_orientation);
        for (std::size_t s = 0; s < 2; ++s)
            values[r * 2 + s] = dof_values[s];
    }
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation,
                                  const ufc::coordinate_mapping * cm=nullptr
                                  ) const final override
  {
    std::size_t num_derivatives = std::pow(2, n);
    std::fill_n(values, 2 * num_derivatives, 0.0);
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
        return;
    }
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
        return;
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    // Compute Inverse Jacobian and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    // Get coordinates and map to the reference (FIAT) element
    double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
    // Precomputed combinations
    const std::size_t combinations[1][2][1] =
        { { { 0 },
            { 1 } } };
    // Declare transformation matrix
    double transform[2][2] =
        { { 1.0, 1.0 },
          { 1.0, 1.0 } };
    // Construct transformation matrix
    for (std::size_t row = 0; row < num_derivatives; ++row)
        for (std::size_t col = 0; col < num_derivatives; ++col)
            for (std::size_t k = 0; k < n; ++k)
                transform[row][col] *= K[2 * combinations[n - 1][col][k] + combinations[n - 1][row][k]];
    switch (i)
    {
    case 0:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 1:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 2:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910316, 0.0, 0.3333333333333333 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 3:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, -0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[num_derivatives + r] += transform[r][s] * derivatives[s];
        }
        break;
    case 4:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910317, 0.2886751345948129, -0.16666666666666666 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[num_derivatives + r] += transform[r][s] * derivatives[s];
        }
        break;
    case 5:
        {
            double basisvalues[3] = {};
            basisvalues[0] = 1.0;
            const double tmp1_1 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_1;
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[1] *= std::sqrt(3.0);
            // Table(s) of coefficients
            static const double coefficients0[3] = { 0.4714045207910316, 0.0, 0.3333333333333333 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 4.8989794855663495, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0 } };
            static const double dmats1[3][3] =
                { { 0.0, 0.0, 0.0 },
                  { 2.449489742783182, 0.0, 0.0 },
                  { 4.242640687119285, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[2] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[3][3] =
                { { 1.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 9, 0.0);
                for (std::size_t t = 0; t < 3; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 9, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 9, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 3; ++t)
                            for (std::size_t u = 0; u < 3; ++u)
                                for (std::size_t tu = 0; tu < 3; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 3; ++s)
                    for (std::size_t t = 0; t < 3; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[num_derivatives + r] += transform[r][s] * derivatives[s];
        }
        break;
    }
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation,
                                      const ufc::coordinate_mapping * cm=nullptr
                                      ) const final override
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
        return;
    }
    unsigned int num_derivatives = std::pow(2, n);
    // Set values equal to zero.
    std::fill_n(values, 2 * num_derivatives * 6, 0.0);
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 1)
        return;
    // Helper variable to hold values of a single dof.
    double dof_values[4] = {};
    // Loop dofs and call evaluate_basis_derivatives.
    for (std::size_t r = 0; r < 6; ++r)
    {
        evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
        for (std::size_t s = 0; s < 2 * num_derivatives; ++s)
            values[2 * num_derivatives * r + s] = dof_values[s];
    }
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    // Declare variables for result of evaluation
    double vals[2];
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
        {
            y[0] = coordinate_dofs[0];
            y[1] = coordinate_dofs[1];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 1:
        {
            y[0] = coordinate_dofs[2];
            y[1] = coordinate_dofs[3];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 2:
        {
            y[0] = coordinate_dofs[4];
            y[1] = coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 3:
        {
            y[0] = coordinate_dofs[0];
            y[1] = coordinate_dofs[1];
            f.evaluate(vals, y, c);
            return vals[1];
        }
        break;
    case 4:
        {
            y[0] = coordinate_dofs[2];
            y[1] = coordinate_dofs[3];
            f.evaluate(vals, y, c);
            return vals[1];
        }
        break;
    case 5:
        {
            y[0] = coordinate_dofs[4];
            y[1] = coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[1];
        }
        break;
    }
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c,
                             const ufc::coordinate_mapping * cm=nullptr
                             ) const final override
  {
    // Declare variables for result of evaluation
    double vals[2];
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[3] = vals[1];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[4] = vals[1];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[5] = vals[1];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::coordinate_mapping * cm=nullptr
                                 ) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[2] = dof_values[1];
    vertex_values[4] = dof_values[2];
    // Evaluate function and change variables
    vertex_values[1] = dof_values[3];
    vertex_values[3] = dof_values[4];
    vertex_values[5] = dof_values[5];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs,
                                const ufc::coordinate_mapping * cm=nullptr
                                ) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[2 + 1] = coordinate_dofs[3];
    dof_coordinates[2 * 2] = coordinate_dofs[4];
    dof_coordinates[2 * 2 + 1] = coordinate_dofs[5];
    dof_coordinates[2 * 3] = coordinate_dofs[0];
    dof_coordinates[2 * 3 + 1] = coordinate_dofs[1];
    dof_coordinates[2 * 4] = coordinate_dofs[2];
    dof_coordinates[2 * 4 + 1] = coordinate_dofs[3];
    dof_coordinates[2 * 5] = coordinate_dofs[4];
    dof_coordinates[2 * 5 + 1] = coordinate_dofs[5];
  }

  void tabulate_reference_dof_coordinates(double * reference_dof_coordinates) const final override
  {
    static const double dof_X[12] = { 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0 };
    std::copy_n(dof_X, 12, reference_dof_coordinates);
  }

  std::size_t num_sub_elements() const final override
  {
    return 2;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_finite_element_0();
    case 1:
        return new biharmonic_finite_element_0();
    default:
        return nullptr;
    }
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_1();
  }

};


class biharmonic_finite_element_2: public ufc::finite_element
{
public:

  biharmonic_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_2() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Real', triangle, 0)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 1;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 0;
  }

  const char * family() const final override
  {
    return "Real";
  }

  void evaluate_reference_basis(double * reference_values,
                                std::size_t num_points,
                                const double * X) const final override
  {
    static const double coefficients0[1][1] = { { 1.0 } };
    for (std::size_t k = 0; k < num_points; ++k)
        reference_values[k] = 0.0;
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Compute basisvalues for each relevant embedded degree
        double basisvalues0[1] = {};
        basisvalues0[0] = 1.0;
        // Accumulate products of coefficients and basisvalues
        reference_values[ip] += coefficients0[0][0] * basisvalues0[0];
    }
  }

  void evaluate_reference_basis_derivatives(double * reference_values,
                                            std::size_t order,
                                            std::size_t num_points,
                                            const double * X) const final override
  {
    if (order == 0)
    {
        evaluate_reference_basis(reference_values, num_points, X);
        return;
    }
    const std::size_t num_derivatives = std::pow(2, order);
    std::fill_n(reference_values, num_points * num_derivatives, 0.0);
    if (order > 0)
        return;
  }

  void transform_reference_basis_derivatives(double * values,
                                             std::size_t order,
                                             std::size_t num_points,
                                             const double * reference_values,
                                             const double * X,
                                             const double * J,
                                             const double * detJ,
                                             const double * K,
                                             int cell_orientation) const final override
  {
    std::fill_n(values, num_points, 0.0);
    const std::size_t reference_offsets[1] = {};
    const std::size_t physical_offsets[1] = {};
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        double transform[1][1];
        for (std::size_t r = 0; r < 1; ++r)
            for (std::size_t s = 0; s < 1; ++s)
                transform[r][s] = 1.0;
        for (std::size_t d = 0; d < 1; ++d)
        {
            for (std::size_t s = 0; s < 1; ++s)
            {
                for (std::size_t i = 0; i < 1; ++i)
                {
                    // Using affine transform to map values back to the physical element.
                    const double mapped_value = reference_values[ip + d + s + reference_offsets[d]];
                    // Mapping derivatives back to the physical element
                    for (std::size_t r = 0; r < 1; ++r)
                        values[ip + d + r + (physical_offsets[d] + i)] += transform[r][s] * mapped_value;
                }
            }
        }
    }
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    double X[2] = {};
    double J[4];
    double detJ;
    double K[4];
    if (cm)
    {
        cm->compute_reference_geometry(X, J, &detJ, K, 1, x, coordinate_dofs, cell_orientation);
    }
    else
    {
        compute_jacobian_triangle_2d(J, coordinate_dofs);
        compute_jacobian_inverse_triangle_2d(K, detJ, J);
    }
    // Evaluate basis on reference element
    double ref_values[1];
    evaluate_reference_basis(ref_values, 1, X);
    // Push forward
    double physical_values[1];
    transform_reference_basis_derivatives(physical_values, 0, 1, ref_values, X, J, &detJ, K, cell_orientation);
    for (std::size_t k = 0; k < 1; ++k)
        values[k] = physical_values[i + k];
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation,
                          const ufc::coordinate_mapping * cm=nullptr
                          ) const final override
  {
    // Element is constant, calling evaluate_basis.
    evaluate_basis(0, values, x, coordinate_dofs, cell_orientation);
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation,
                                  const ufc::coordinate_mapping * cm=nullptr
                                  ) const final override
  {
    std::size_t num_derivatives = std::pow(2, n);
    std::fill_n(values, num_derivatives, 0.0);
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
        return;
    }
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation,
                                      const ufc::coordinate_mapping * cm=nullptr
                                      ) const final override
  {
    // Element is constant, calling evaluate_basis_derivatives.
    evaluate_basis_derivatives(0, n, values, x, coordinate_dofs, cell_orientation);
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
        {
            y[0] = 0.3333333333333334 * coordinate_dofs[0] + 0.3333333333333333 * coordinate_dofs[2] + 0.3333333333333333 * coordinate_dofs[4];
            y[1] = 0.3333333333333334 * coordinate_dofs[1] + 0.3333333333333333 * coordinate_dofs[3] + 0.3333333333333333 * coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    }
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c,
                             const ufc::coordinate_mapping * cm=nullptr
                             ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    y[0] = 0.3333333333333334 * coordinate_dofs[0] + 0.3333333333333333 * coordinate_dofs[2] + 0.3333333333333333 * coordinate_dofs[4];
    y[1] = 0.3333333333333334 * coordinate_dofs[1] + 0.3333333333333333 * coordinate_dofs[3] + 0.3333333333333333 * coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::coordinate_mapping * cm=nullptr
                                 ) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs,
                                const ufc::coordinate_mapping * cm=nullptr
                                ) const final override
  {
    dof_coordinates[0] = 0.3333333333333334 * coordinate_dofs[0] + 0.3333333333333333 * coordinate_dofs[2] + 0.3333333333333333 * coordinate_dofs[4];
    dof_coordinates[1] = 0.3333333333333334 * coordinate_dofs[1] + 0.3333333333333333 * coordinate_dofs[3] + 0.3333333333333333 * coordinate_dofs[5];
  }

  void tabulate_reference_dof_coordinates(double * reference_dof_coordinates) const final override
  {
    static const double dof_X[2] = { 0.3333333333333333, 0.3333333333333333 };
    std::copy_n(dof_X, 2, reference_dof_coordinates);
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_2();
  }

};


class biharmonic_finite_element_3: public ufc::finite_element
{
public:

  biharmonic_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  ~biharmonic_finite_element_3() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FiniteElement('Lagrange', triangle, 2)";
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t space_dimension() const final override
  {
    return 6;
  }

  std::size_t value_rank() const final override
  {
    return 0;
  }

  std::size_t value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t value_size() const final override
  {
    return 1;
  }

  std::size_t reference_value_rank() const final override
  {
    return 0;
  }

  std::size_t reference_value_dimension(std::size_t i) const final override
  {
    return 1;
  }

  std::size_t reference_value_size() const final override
  {
    return 1;
  }

  std::size_t degree() const final override
  {
    return 2;
  }

  const char * family() const final override
  {
    return "Lagrange";
  }

  void evaluate_reference_basis(double * reference_values,
                                std::size_t num_points,
                                const double * X) const final override
  {
    static const double coefficients0[1][6] = { { 0.0, -0.17320508075688776, -0.09999999999999999, 0.12171612389003691, 0.09428090415820634, 0.05443310539518172 } };
    static const double coefficients1[1][6] = { { 0.0, 0.1732050807568877, -0.09999999999999999, 0.12171612389003691, -0.09428090415820636, 0.05443310539518175 } };
    static const double coefficients2[1][6] = { { 0.0, 0.0, 0.20000000000000004, 0.0, 0.0, 0.16329931618554522 } };
    static const double coefficients3[1][6] = { { 0.47140452079103157, 0.23094010767585027, 0.1333333333333333, 0.0, 0.18856180831641264, -0.1632993161855452 } };
    static const double coefficients4[1][6] = { { 0.47140452079103157, -0.23094010767585027, 0.13333333333333333, 0.0, -0.18856180831641264, -0.16329931618554522 } };
    static const double coefficients5[1][6] = { { 0.4714045207910316, 0.0, -0.26666666666666666, -0.24343224778007383, 0.0, 0.05443310539518172 } };
    for (std::size_t k = 0; k < num_points * 6; ++k)
        reference_values[k] = 0.0;
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues2[6] = {};
        basisvalues2[0] = 1.0;
        const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues2[1] = tmp1_2;
        const double tmp2_2 = (1.0 - Y[1]) / 2.0;
        const double tmp3_2 = tmp2_2 * tmp2_2;
        basisvalues2[3] = 1.5 * tmp1_2 * basisvalues2[1] - 0.5 * tmp3_2 * basisvalues2[0];
        basisvalues2[2] = (0.5 + 1.5 * Y[1]) * basisvalues2[0];
        basisvalues2[4] = (1.5 + 2.5 * Y[1]) * basisvalues2[1];
        basisvalues2[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues2[2] - 0.5555555555555556 * basisvalues2[0];
        basisvalues2[0] *= std::sqrt(0.5);
        basisvalues2[2] *= std::sqrt(1.0);
        basisvalues2[5] *= std::sqrt(1.5);
        basisvalues2[1] *= std::sqrt(3.0);
        basisvalues2[4] *= std::sqrt(4.5);
        basisvalues2[3] *= std::sqrt(7.5);
        // Accumulate products of coefficients and basisvalues
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip] += coefficients0[0][r] * basisvalues2[r];
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip + 1] += coefficients1[0][r] * basisvalues2[r];
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip + 2] += coefficients2[0][r] * basisvalues2[r];
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip + 3] += coefficients3[0][r] * basisvalues2[r];
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip + 4] += coefficients4[0][r] * basisvalues2[r];
        for (std::size_t r = 0; r < 6; ++r)
            reference_values[6 * ip + 5] += coefficients5[0][r] * basisvalues2[r];
    }
  }

  void evaluate_reference_basis_derivatives(double * reference_values,
                                            std::size_t order,
                                            std::size_t num_points,
                                            const double * X) const final override
  {
    if (order == 0)
    {
        evaluate_reference_basis(reference_values, num_points, X);
        return;
    }
    const std::size_t num_derivatives = std::pow(2, order);
    std::fill_n(reference_values, num_points * 6 * num_derivatives, 0.0);
    if (order > 2)
        return;
    // Tables of derivatives of the polynomial base (transpose).
    alignas(32) static const double dmats0[2][6][6] =
        { { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
            { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
            { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
          { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
            { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
            { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
            { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } } };
    static const double coefficients0[1][6] = { { 0.0, -0.17320508075688776, -0.09999999999999999, 0.12171612389003691, 0.09428090415820634, 0.05443310539518172 } };
    static const double coefficients1[1][6] = { { 0.0, 0.1732050807568877, -0.09999999999999999, 0.12171612389003691, -0.09428090415820636, 0.05443310539518175 } };
    static const double coefficients2[1][6] = { { 0.0, 0.0, 0.20000000000000004, 0.0, 0.0, 0.16329931618554522 } };
    static const double coefficients3[1][6] = { { 0.47140452079103157, 0.23094010767585027, 0.1333333333333333, 0.0, 0.18856180831641264, -0.1632993161855452 } };
    static const double coefficients4[1][6] = { { 0.47140452079103157, -0.23094010767585027, 0.13333333333333333, 0.0, -0.18856180831641264, -0.16329931618554522 } };
    static const double coefficients5[1][6] = { { 0.4714045207910316, 0.0, -0.26666666666666666, -0.24343224778007383, 0.0, 0.05443310539518172 } };
    const std::size_t reference_offset[6] = {};
    const std::size_t num_components[6] = { 1, 1, 1, 1, 1, 1 };
    // Precomputed combinations
    const std::size_t combinations[2][4][2] =
        { { { 0, 0 },
            { 1, 0 },
            { 0, 0 },
            { 1, 0 } },
          { { 0, 0 },
            { 0, 1 },
            { 1, 0 },
            { 1, 1 } } };
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Map from UFC reference coordinate X to FIAT reference coordinate Y
        const double Y[2] = { 2.0 * X[ip * 2] - 1.0, 2.0 * X[ip * 2 + 1] - 1.0 };
        // Compute basisvalues for each relevant embedded degree
        double basisvalues2[6] = {};
        basisvalues2[0] = 1.0;
        const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
        basisvalues2[1] = tmp1_2;
        const double tmp2_2 = (1.0 - Y[1]) / 2.0;
        const double tmp3_2 = tmp2_2 * tmp2_2;
        basisvalues2[3] = 1.5 * tmp1_2 * basisvalues2[1] - 0.5 * tmp3_2 * basisvalues2[0];
        basisvalues2[2] = (0.5 + 1.5 * Y[1]) * basisvalues2[0];
        basisvalues2[4] = (1.5 + 2.5 * Y[1]) * basisvalues2[1];
        basisvalues2[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues2[2] - 0.5555555555555556 * basisvalues2[0];
        basisvalues2[0] *= std::sqrt(0.5);
        basisvalues2[2] *= std::sqrt(1.0);
        basisvalues2[5] *= std::sqrt(1.5);
        basisvalues2[1] *= std::sqrt(3.0);
        basisvalues2[4] *= std::sqrt(4.5);
        basisvalues2[3] *= std::sqrt(7.5);
        // Loop over all dofs
        for (std::size_t i = 0; i < 6; ++i)
        {
            double derivatives[4] = {};
            switch (i)
            {
            case 0:
                // Compute reference derivatives for dof 0.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients0[0][s] * aux[s];
                }
                break;
            case 1:
                // Compute reference derivatives for dof 1.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients1[0][s] * aux[s];
                }
                break;
            case 2:
                // Compute reference derivatives for dof 2.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients2[0][s] * aux[s];
                }
                break;
            case 3:
                // Compute reference derivatives for dof 3.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients3[0][s] * aux[s];
                }
                break;
            case 4:
                // Compute reference derivatives for dof 4.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients4[0][s] * aux[s];
                }
                break;
            case 5:
                // Compute reference derivatives for dof 5.
                for (std::size_t r = 0; r < num_derivatives; ++r)
                {
                    double aux[6] = {};
                    // Declare derivative matrix (of polynomial basis).
                    double dmats[6][6] = {};
                    // Initialize dmats.
                    std::size_t comb = combinations[order - 1][r][0];
                    std::copy_n(&dmats0[comb][0][0], 36, &dmats[0][0]);
                    // Looping derivative order to generate dmats.
                    for (std::size_t s = 1; s < order; ++s)
                    {
                        // Store previous dmats matrix.
                        double dmats_old[6][6];
                        std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                        // Resetting dmats.
                        std::fill_n(&dmats[0][0], 36, 0.0);
                        // Update dmats using an inner product.
                        comb = combinations[order - 1][r][s];
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats0[comb][t][tu] * dmats_old[tu][u];
                    }
                    for (std::size_t s = 0; s < 6; ++s)
                        for (std::size_t t = 0; t < 6; ++t)
                            aux[s] += dmats[s][t] * basisvalues2[t];
                    derivatives[r] = 0.0;
                    for (std::size_t s = 0; s < 6; ++s)
                        derivatives[r] += coefficients5[0][s] * aux[s];
                }
                break;
            }
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t c = 0; c < num_components[i]; ++c)
                    reference_values[6 * num_derivatives * ip + num_derivatives * i + r + (reference_offset[i] + c)] = derivatives[num_derivatives * c + r];
        }
    }
  }

  void transform_reference_basis_derivatives(double * values,
                                             std::size_t order,
                                             std::size_t num_points,
                                             const double * reference_values,
                                             const double * X,
                                             const double * J,
                                             const double * detJ,
                                             const double * K,
                                             int cell_orientation) const final override
  {
    const std::size_t num_derivatives = std::pow(2, order);
    // Precomputed combinations
    const std::size_t combinations[2][4][2] =
        { { { 0, 0 },
            { 1, 0 },
            { 0, 0 },
            { 1, 0 } },
          { { 0, 0 },
            { 0, 1 },
            { 1, 0 },
            { 1, 1 } } };
    std::fill_n(values, num_points * 6 * num_derivatives, 0.0);
    const std::size_t reference_offsets[6] = {};
    const std::size_t physical_offsets[6] = {};
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        double transform[4][4];
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                transform[r][s] = 1.0;
        for (std::size_t r = 0; r < num_derivatives; ++r)
            for (std::size_t s = 0; s < num_derivatives; ++s)
                for (std::size_t k = 0; k < order; ++k)
                    transform[r][s] *= K[2 * 2 * ip + 2 * combinations[order - 1][s][k] + combinations[order - 1][r][k]];
        for (std::size_t d = 0; d < 6; ++d)
        {
            for (std::size_t s = 0; s < num_derivatives; ++s)
            {
                for (std::size_t i = 0; i < 1; ++i)
                {
                    // Using affine transform to map values back to the physical element.
                    const double mapped_value = reference_values[6 * num_derivatives * ip + num_derivatives * d + s + reference_offsets[d]];
                    // Mapping derivatives back to the physical element
                    for (std::size_t r = 0; r < num_derivatives; ++r)
                        values[6 * num_derivatives * ip + num_derivatives * d + r + (physical_offsets[d] + i)] += transform[r][s] * mapped_value;
                }
            }
        }
    }
  }

  void evaluate_basis(std::size_t i,
                      double * values,
                      const double * x,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    double X[2] = {};
    double J[4];
    double detJ;
    double K[4];
    if (cm)
    {
        cm->compute_reference_geometry(X, J, &detJ, K, 1, x, coordinate_dofs, cell_orientation);
    }
    else
    {
        compute_jacobian_triangle_2d(J, coordinate_dofs);
        compute_jacobian_inverse_triangle_2d(K, detJ, J);
        // Compute constants
        const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
        const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
        // Get coordinates and map to the reference (FIAT) element
        double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
        // Map to FFC reference coordinate
        for (std::size_t k = 0; k < 2; ++k)
            X[k] = (Y[k] + 1.0) / 2.0;
    }
    // Evaluate basis on reference element
    double ref_values[6];
    evaluate_reference_basis(ref_values, 1, X);
    // Push forward
    double physical_values[6];
    transform_reference_basis_derivatives(physical_values, 0, 1, ref_values, X, J, &detJ, K, cell_orientation);
    for (std::size_t k = 0; k < 1; ++k)
        values[k] = physical_values[i + k];
  }

  void evaluate_basis_all(double * values,
                          const double * x,
                          const double * coordinate_dofs,
                          int cell_orientation,
                          const ufc::coordinate_mapping * cm=nullptr
                          ) const final override
  {
    // Helper variable to hold value of a single dof.
    double dof_values = 0.0;
    // Loop dofs and call evaluate_basis
    for (std::size_t r = 0; r < 6; ++r)
    {
        evaluate_basis(r, &dof_values, x, coordinate_dofs, cell_orientation);
        values[r] = dof_values;
    }
  }

  void evaluate_basis_derivatives(std::size_t i,
                                  std::size_t n,
                                  double * values,
                                  const double * x,
                                  const double * coordinate_dofs,
                                  int cell_orientation,
                                  const ufc::coordinate_mapping * cm=nullptr
                                  ) const final override
  {
    std::size_t num_derivatives = std::pow(2, n);
    std::fill_n(values, num_derivatives, 0.0);
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis(i, values, x, coordinate_dofs, cell_orientation);
        return;
    }
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 2)
        return;
    // Compute Jacobian
    double J[4];
    compute_jacobian_triangle_2d(J, coordinate_dofs);
    // Compute Inverse Jacobian and determinant
    double K[4];
    double detJ;
    compute_jacobian_inverse_triangle_2d(K, detJ, J);
    // Compute constants
    const double C0 = coordinate_dofs[2] + coordinate_dofs[4];
    const double C1 = coordinate_dofs[3] + coordinate_dofs[5];
    // Get coordinates and map to the reference (FIAT) element
    double Y[2] = { (J[1] * (C1 - 2.0 * x[1]) + J[3] * (2.0 * x[0] - C0)) / detJ, (J[0] * (2.0 * x[1] - C1) + J[2] * (C0 - 2.0 * x[0])) / detJ };
    // Precomputed combinations
    const std::size_t combinations[2][4][2] =
        { { { 0, 0 },
            { 1, 0 },
            { 0, 0 },
            { 1, 0 } },
          { { 0, 0 },
            { 0, 1 },
            { 1, 0 },
            { 1, 1 } } };
    // Declare transformation matrix
    double transform[4][4] =
        { { 1.0, 1.0, 1.0, 1.0 },
          { 1.0, 1.0, 1.0, 1.0 },
          { 1.0, 1.0, 1.0, 1.0 },
          { 1.0, 1.0, 1.0, 1.0 } };
    // Construct transformation matrix
    for (std::size_t row = 0; row < num_derivatives; ++row)
        for (std::size_t col = 0; col < num_derivatives; ++col)
            for (std::size_t k = 0; k < n; ++k)
                transform[row][col] *= K[2 * combinations[n - 1][col][k] + combinations[n - 1][row][k]];
    switch (i)
    {
    case 0:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.0, -0.17320508075688776, -0.09999999999999999, 0.12171612389003691, 0.09428090415820634, 0.05443310539518172 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 1:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.0, 0.1732050807568877, -0.09999999999999999, 0.12171612389003691, -0.09428090415820636, 0.05443310539518175 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 2:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.0, 0.0, 0.20000000000000004, 0.0, 0.0, 0.16329931618554522 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 3:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.47140452079103157, 0.23094010767585027, 0.1333333333333333, 0.0, 0.18856180831641264, -0.1632993161855452 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 4:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.47140452079103157, -0.23094010767585027, 0.13333333333333333, 0.0, -0.18856180831641264, -0.16329931618554522 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    case 5:
        {
            double basisvalues[6] = {};
            basisvalues[0] = 1.0;
            const double tmp1_2 = (1.0 + 2.0 * Y[0] + Y[1]) / 2.0;
            basisvalues[1] = tmp1_2;
            const double tmp2_2 = (1.0 - Y[1]) / 2.0;
            const double tmp3_2 = tmp2_2 * tmp2_2;
            basisvalues[3] = 1.5 * tmp1_2 * basisvalues[1] - 0.5 * tmp3_2 * basisvalues[0];
            basisvalues[2] = (0.5 + 1.5 * Y[1]) * basisvalues[0];
            basisvalues[4] = (1.5 + 2.5 * Y[1]) * basisvalues[1];
            basisvalues[5] = (0.1111111111111111 + 1.6666666666666667 * Y[1]) * basisvalues[2] - 0.5555555555555556 * basisvalues[0];
            basisvalues[0] *= std::sqrt(0.5);
            basisvalues[2] *= std::sqrt(1.0);
            basisvalues[5] *= std::sqrt(1.5);
            basisvalues[1] *= std::sqrt(3.0);
            basisvalues[4] *= std::sqrt(4.5);
            basisvalues[3] *= std::sqrt(7.5);
            // Table(s) of coefficients
            static const double coefficients0[6] = { 0.4714045207910316, 0.0, -0.26666666666666666, -0.24343224778007383, 0.0, 0.05443310539518172 };
            // Tables of derivatives of the polynomial base (transpose).
            static const double dmats0[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.898979485566347, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 9.486832980505149, 0.0, 0.0, 0.0, 0.0 },
                  { 3.999999999999994, 0.0, 7.0710678118654675, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } };
            static const double dmats1[6][6] =
                { { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.4494897427831805, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 4.242640687119282, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 2.5819888974716183, 4.743416490252574, -0.9128709291752797, 0.0, 0.0, 0.0 },
                  { 1.9999999999999125, 6.123724356957938, 3.5355339059327333, 0.0, 0.0, 0.0 },
                  { -2.3094010767584887, 0.0, 8.164965809277266, 0.0, 0.0, 0.0 } };
            // Compute reference derivatives.
            // Declare array of derivatives on FIAT element.
            double derivatives[4] = {};
            // Declare derivative matrix (of polynomial basis).
            double dmats[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Declare (auxiliary) derivative matrix (of polynomial basis).
            double dmats_old[6][6] =
                { { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 },
                  { 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 } };
            // Loop possible derivatives.
            for (std::size_t r = 0; r < num_derivatives; ++r)
            {
                // Reset dmats to identity
                std::fill_n(&dmats[0][0], 36, 0.0);
                for (std::size_t t = 0; t < 6; ++t)
                    dmats[t][t] = 1.0;
                // Looping derivative order to generate dmats.
                for (std::size_t s = 0; s < n; ++s)
                {
                    std::copy_n(&dmats[0][0], 36, &dmats_old[0][0]);
                    std::fill_n(&dmats[0][0], 36, 0.0);
                    // Update dmats using an inner product.
                    // _dmats_product(shape_dmats, comb[r][s], 0)
                    if (combinations[n - 1][r][s] == 0)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats0[t][tu];
                    }
                    // _dmats_product(shape_dmats, comb[r][s], 1)
                    if (combinations[n - 1][r][s] == 1)
                    {
                        for (std::size_t t = 0; t < 6; ++t)
                            for (std::size_t u = 0; u < 6; ++u)
                                for (std::size_t tu = 0; tu < 6; ++tu)
                                    dmats[t][u] += dmats_old[tu][u] * dmats1[t][tu];
                    }
                }
                for (std::size_t s = 0; s < 6; ++s)
                    for (std::size_t t = 0; t < 6; ++t)
                        derivatives[r] += coefficients0[s] * dmats[s][t] * basisvalues[t];
            }
            // Transform derivatives back to physical element
            for (std::size_t r = 0; r < num_derivatives; ++r)
                for (std::size_t s = 0; s < num_derivatives; ++s)
                    values[r] += transform[r][s] * derivatives[s];
        }
        break;
    }
  }

  void evaluate_basis_derivatives_all(std::size_t n,
                                      double * values,
                                      const double * x,
                                      const double * coordinate_dofs,
                                      int cell_orientation,
                                      const ufc::coordinate_mapping * cm=nullptr
                                      ) const final override
  {
    // Call evaluate_basis_all if order of derivatives is equal to zero.
    if (n == 0)
    {
        evaluate_basis_all(values, x, coordinate_dofs, cell_orientation);
        return;
    }
    unsigned int num_derivatives = std::pow(2, n);
    // Set values equal to zero.
    std::fill_n(values, num_derivatives * 6, 0.0);
    // If order of derivatives is greater than the maximum polynomial degree, return zeros.
    if (n > 2)
        return;
    // Helper variable to hold values of a single dof.
    double dof_values[4] = {};
    // Loop dofs and call evaluate_basis_derivatives.
    for (std::size_t r = 0; r < 6; ++r)
    {
        evaluate_basis_derivatives(r, n, dof_values, x, coordinate_dofs, cell_orientation);
        for (std::size_t s = 0; s < num_derivatives; ++s)
            values[num_derivatives * r + s] = dof_values[s];
    }
  }

  double evaluate_dof(std::size_t i,
                      const ufc::function& f,
                      const double * coordinate_dofs,
                      int cell_orientation,
                      const ufc::cell& c,
                      const ufc::coordinate_mapping * cm=nullptr
                      ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    switch (i)
    {
    case 0:
        {
            y[0] = coordinate_dofs[0];
            y[1] = coordinate_dofs[1];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 1:
        {
            y[0] = coordinate_dofs[2];
            y[1] = coordinate_dofs[3];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 2:
        {
            y[0] = coordinate_dofs[4];
            y[1] = coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 3:
        {
            y[0] = 0.5 * coordinate_dofs[2] + 0.5 * coordinate_dofs[4];
            y[1] = 0.5 * coordinate_dofs[3] + 0.5 * coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 4:
        {
            y[0] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[4];
            y[1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[5];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    case 5:
        {
            y[0] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[2];
            y[1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[3];
            f.evaluate(vals, y, c);
            return vals[0];
        }
        break;
    }
    return 0.0;
  }

  void evaluate_dofs(double * values,
                             const ufc::function& f,
                             const double * coordinate_dofs,
                             int cell_orientation,
                             const ufc::cell& c,
                             const ufc::coordinate_mapping * cm=nullptr
                             ) const final override
  {
    // Declare variables for result of evaluation
    double vals[1];
    // Declare variable for physical coordinates
    double y[2];
    y[0] = coordinate_dofs[0];
    y[1] = coordinate_dofs[1];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = coordinate_dofs[2];
    y[1] = coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = coordinate_dofs[4];
    y[1] = coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = 0.5 * coordinate_dofs[2] + 0.5 * coordinate_dofs[4];
    y[1] = 0.5 * coordinate_dofs[3] + 0.5 * coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[4];
    y[1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[5];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[2];
    y[1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[3];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
  }

  void interpolate_vertex_values(double * vertex_values,
                                 const double * dof_values,
                                 const double * coordinate_dofs,
                                 int cell_orientation,
                                 const ufc::coordinate_mapping * cm=nullptr
                                 ) const final override
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  void tabulate_dof_coordinates(double * dof_coordinates,
                                const double * coordinate_dofs,
                                const ufc::coordinate_mapping * cm=nullptr
                                ) const final override
  {
    dof_coordinates[0] = coordinate_dofs[0];
    dof_coordinates[1] = coordinate_dofs[1];
    dof_coordinates[2] = coordinate_dofs[2];
    dof_coordinates[2 + 1] = coordinate_dofs[3];
    dof_coordinates[2 * 2] = coordinate_dofs[4];
    dof_coordinates[2 * 2 + 1] = coordinate_dofs[5];
    dof_coordinates[2 * 3] = 0.5 * coordinate_dofs[2] + 0.5 * coordinate_dofs[4];
    dof_coordinates[2 * 3 + 1] = 0.5 * coordinate_dofs[3] + 0.5 * coordinate_dofs[5];
    dof_coordinates[2 * 4] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[4];
    dof_coordinates[2 * 4 + 1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[5];
    dof_coordinates[2 * 5] = 0.5 * coordinate_dofs[0] + 0.5 * coordinate_dofs[2];
    dof_coordinates[2 * 5 + 1] = 0.5 * coordinate_dofs[1] + 0.5 * coordinate_dofs[3];
  }

  void tabulate_reference_dof_coordinates(double * reference_dof_coordinates) const final override
  {
    static const double dof_X[12] = { 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.5, 0.5, 0.0, 0.5, 0.5, 0.0 };
    std::copy_n(dof_X, 12, reference_dof_coordinates);
  }

  std::size_t num_sub_elements() const final override
  {
    return 0;
  }

  ufc::finite_element * create_sub_element(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::finite_element * create() const final override
  {
    return new biharmonic_finite_element_3();
  }

};


class biharmonic_dofmap_0: public ufc::dofmap
{
public:

  biharmonic_dofmap_0() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Lagrange', triangle, 1)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    static const bool return_values[3] = { true, false, false };
    if (d >= 3)
        return false;
    return return_values[d];
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return num_global_entities[0];
  }

  std::size_t num_global_support_dofs() const final override
  {
    return 0;
  }

  std::size_t num_element_support_dofs() const final override
  {
    return 3;
  }

  std::size_t num_element_dofs() const final override
  {
    return 3;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 2;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 1, 0, 0 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 1, 2, 3 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    dofs[0] = entity_indices[0][0];
    dofs[1] = entity_indices[0][1];
    dofs[2] = entity_indices[0][2];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
        dofs[0] = 1;
        dofs[1] = 2;
        break;
    case 1:
        dofs[0] = 0;
        dofs[1] = 2;
        break;
    case 2:
        dofs[0] = 0;
        dofs[1] = 1;
        break;
    }
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        case 1:
            dofs[0] = 1;
            break;
        case 2:
            dofs[0] = 2;
            break;
        }
        break;
    }
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        case 1:
            dofs[0] = 1;
            break;
        case 2:
            dofs[0] = 2;
            break;
        }
        break;
    case 1:
        switch (i)
        {
        case 0:
            dofs[0] = 1;
            dofs[1] = 2;
            break;
        case 1:
            dofs[0] = 0;
            dofs[1] = 2;
            break;
        case 2:
            dofs[0] = 0;
            dofs[1] = 1;
            break;
        }
        break;
    case 2:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            dofs[1] = 1;
            dofs[2] = 2;
            break;
        }
        break;
    }
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_0();
  }

};


class biharmonic_dofmap_1: public ufc::dofmap
{
public:

  biharmonic_dofmap_1() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for VectorElement(FiniteElement('Lagrange', triangle, 1), dim=2)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    static const bool return_values[3] = { true, false, false };
    if (d >= 3)
        return false;
    return return_values[d];
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return 2 * num_global_entities[0];
  }

  std::size_t num_global_support_dofs() const final override
  {
    return 0;
  }

  std::size_t num_element_support_dofs() const final override
  {
    return 6;
  }

  std::size_t num_element_dofs() const final override
  {
    return 6;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 4;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 2, 0, 0 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 2, 4, 6 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    std::size_t offset = 0;
    dofs[0] = offset + entity_indices[0][0];
    dofs[1] = offset + entity_indices[0][1];
    dofs[2] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
    dofs[3] = offset + entity_indices[0][0];
    dofs[4] = offset + entity_indices[0][1];
    dofs[5] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
        dofs[0] = 1;
        dofs[1] = 2;
        dofs[2] = 4;
        dofs[3] = 5;
        break;
    case 1:
        dofs[0] = 0;
        dofs[1] = 2;
        dofs[2] = 3;
        dofs[3] = 5;
        break;
    case 2:
        dofs[0] = 0;
        dofs[1] = 1;
        dofs[2] = 3;
        dofs[3] = 4;
        break;
    }
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            dofs[1] = 3;
            break;
        case 1:
            dofs[0] = 1;
            dofs[1] = 4;
            break;
        case 2:
            dofs[0] = 2;
            dofs[1] = 5;
            break;
        }
        break;
    }
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            dofs[1] = 3;
            break;
        case 1:
            dofs[0] = 1;
            dofs[1] = 4;
            break;
        case 2:
            dofs[0] = 2;
            dofs[1] = 5;
            break;
        }
        break;
    case 1:
        switch (i)
        {
        case 0:
            dofs[0] = 1;
            dofs[1] = 2;
            dofs[2] = 4;
            dofs[3] = 5;
            break;
        case 1:
            dofs[0] = 0;
            dofs[1] = 2;
            dofs[2] = 3;
            dofs[3] = 5;
            break;
        case 2:
            dofs[0] = 0;
            dofs[1] = 1;
            dofs[2] = 3;
            dofs[3] = 4;
            break;
        }
        break;
    case 2:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            dofs[1] = 1;
            dofs[2] = 2;
            dofs[3] = 3;
            dofs[4] = 4;
            dofs[5] = 5;
            break;
        }
        break;
    }
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 2;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_dofmap_0();
    case 1:
        return new biharmonic_dofmap_0();
    default:
        return nullptr;
    }
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_1();
  }

};


class biharmonic_dofmap_2: public ufc::dofmap
{
public:

  biharmonic_dofmap_2() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_2() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Real', triangle, 0)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    static const bool return_values[3] = {};
    if (d >= 3)
        return false;
    return return_values[d];
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return 1;
  }

  std::size_t num_global_support_dofs() const final override
  {
    return 1;
  }

  std::size_t num_element_support_dofs() const final override
  {
    return 0;
  }

  std::size_t num_element_dofs() const final override
  {
    return 1;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 0;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 0, 0, 1 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 0, 0, 1 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    dofs[0] = 0;
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    // Do nothing
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 2:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        }
        break;
    }
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            break;
        case 1:
            break;
        case 2:
            break;
        }
        break;
    case 1:
        switch (i)
        {
        case 0:
            break;
        case 1:
            break;
        case 2:
            break;
        }
        break;
    case 2:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        }
        break;
    }
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_2();
  }

};


class biharmonic_dofmap_3: public ufc::dofmap
{
public:

  biharmonic_dofmap_3() : ufc::dofmap()
  {
    // Do nothing
  }

  ~biharmonic_dofmap_3() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC dofmap for FiniteElement('Lagrange', triangle, 2)";
  }

  bool needs_mesh_entities(std::size_t d) const final override
  {
    static const bool return_values[3] = { true, true, false };
    if (d >= 3)
        return false;
    return return_values[d];
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  std::size_t global_dimension(const std::vector<std::size_t>&
                               num_global_entities) const final override
  {
    return num_global_entities[0] + num_global_entities[1];
  }

  std::size_t num_global_support_dofs() const final override
  {
    return 0;
  }

  std::size_t num_element_support_dofs() const final override
  {
    return 6;
  }

  std::size_t num_element_dofs() const final override
  {
    return 6;
  }

  std::size_t num_facet_dofs() const final override
  {
    return 3;
  }

  std::size_t num_entity_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 1, 1, 0 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  std::size_t num_entity_closure_dofs(std::size_t d) const final override
  {
    static const std::size_t return_values[3] = { 1, 3, 6 };
    if (d >= 3)
        return 0;
    return return_values[d];
  }

  void tabulate_dofs(std::size_t * dofs,
                     const std::vector<std::size_t>& num_global_entities,
                     const std::vector<std::vector<std::size_t>>& entity_indices) const final override
  {
    std::size_t offset = 0;
    dofs[0] = offset + entity_indices[0][0];
    dofs[1] = offset + entity_indices[0][1];
    dofs[2] = offset + entity_indices[0][2];
    offset += num_global_entities[0];
    dofs[3] = offset + entity_indices[1][0];
    dofs[4] = offset + entity_indices[1][1];
    dofs[5] = offset + entity_indices[1][2];
    offset += num_global_entities[1];
  }

  void tabulate_facet_dofs(std::size_t * dofs,
                           std::size_t facet) const final override
  {
    switch (facet)
    {
    case 0:
        dofs[0] = 1;
        dofs[1] = 2;
        dofs[2] = 3;
        break;
    case 1:
        dofs[0] = 0;
        dofs[1] = 2;
        dofs[2] = 4;
        break;
    case 2:
        dofs[0] = 0;
        dofs[1] = 1;
        dofs[2] = 5;
        break;
    }
  }

  void tabulate_entity_dofs(std::size_t * dofs,
                            std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        case 1:
            dofs[0] = 1;
            break;
        case 2:
            dofs[0] = 2;
            break;
        }
        break;
    case 1:
        switch (i)
        {
        case 0:
            dofs[0] = 3;
            break;
        case 1:
            dofs[0] = 4;
            break;
        case 2:
            dofs[0] = 5;
            break;
        }
        break;
    }
  }

  void tabulate_entity_closure_dofs(std::size_t * dofs,
                                    std::size_t d, std::size_t i) const final override
  {
    switch (d)
    {
    case 0:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            break;
        case 1:
            dofs[0] = 1;
            break;
        case 2:
            dofs[0] = 2;
            break;
        }
        break;
    case 1:
        switch (i)
        {
        case 0:
            dofs[0] = 1;
            dofs[1] = 2;
            dofs[2] = 3;
            break;
        case 1:
            dofs[0] = 0;
            dofs[1] = 2;
            dofs[2] = 4;
            break;
        case 2:
            dofs[0] = 0;
            dofs[1] = 1;
            dofs[2] = 5;
            break;
        }
        break;
    case 2:
        switch (i)
        {
        case 0:
            dofs[0] = 0;
            dofs[1] = 1;
            dofs[2] = 2;
            dofs[3] = 3;
            dofs[4] = 4;
            dofs[5] = 5;
            break;
        }
        break;
    }
  }


  std::size_t num_sub_dofmaps() const final override
  {
    return 0;
  }

  ufc::dofmap * create_sub_dofmap(std::size_t i) const final override
  {
    return nullptr;
  }

  ufc::dofmap * create() const final override
  {
    return new biharmonic_dofmap_3();
  }

};


class biharmonic_coordinate_mapping_1: public ufc::coordinate_mapping
{
public:

  biharmonic_coordinate_mapping_1() : ufc::coordinate_mapping()
  {
    // Do nothing
  }

  ~biharmonic_coordinate_mapping_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "FFC coordinate_mapping from VectorElement(FiniteElement('Lagrange', triangle, 1), dim=2)";
  }

  ufc::coordinate_mapping * create() const final override
  {
    return new biharmonic_coordinate_mapping_1();
  }

  std::size_t geometric_dimension() const final override
  {
    return 2;
  }

  std::size_t topological_dimension() const final override
  {
    return 2;
  }

  ufc::shape cell_shape() const final override
  {
    return ufc::shape::triangle;
  }

  ufc::finite_element * create_coordinate_finite_element() const final override
  {
    return new biharmonic_finite_element_1();
  }

  ufc::dofmap * create_coordinate_dofmap() const final override
  {
    return new biharmonic_dofmap_1();
  }

  void compute_physical_coordinates(
      double * x, std::size_t num_points,
      const double * X,
      const double * coordinate_dofs) const final override
  {
    biharmonic_finite_element_0 xelement;
    double phi[3];
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Compute basis values of coordinate element
        xelement.evaluate_reference_basis(phi, 1, &X[2 * ip]);
        // Compute x
        for (std::size_t i = 0; i < 2; ++i)
            for (std::size_t d = 0; d < 3; ++d)
                x[2 * ip + i] += coordinate_dofs[2 * d + i] * phi[d];
    }
  }

  void compute_reference_coordinates(
      double * X, std::size_t num_points,
      const double * x,
      const double * coordinate_dofs, int cell_orientation) const final override
  {
    const double phi_X0[3] = { 1.0, 0.0, 0.0 };
    const double dphi_X0[2][3] =
        { { -1.0, 1.0, 0.0 },
          { -1.0, 0.0, 1.0 } };
    double J[4];
    double detJ[1];
    double K[4];
    double x0[2] = {};
    for (std::size_t i = 0; i < 2; ++i)
        for (std::size_t k = 0; k < 3; ++k)
            x0[i] += coordinate_dofs[2 * k + i] * phi_X0[k];
    for (std::size_t i = 0; i < 2; ++i)
    {
        for (std::size_t j = 0; j < 2; ++j)
        {
            J[2 * i + j] = 0.0;
            for (std::size_t k = 0; k < 3; ++k)
                J[2 * i + j] += coordinate_dofs[2 * k + i] * dphi_X0[j][k];
        }
    }
    compute_jacobian_determinants(detJ, 1, J, cell_orientation);
    compute_jacobian_inverses(K, 1, J, detJ);
    for (std::size_t ip = 0; ip < num_points; ++ip)
        for (std::size_t j = 0; j < 2; ++j)
            for (std::size_t i = 0; i < 2; ++i)
                X[2 * ip + j] += K[2 * j + i] * (x[2 * ip + i] - x0[i]);
  }

  void compute_reference_geometry(
      double * X, double * J, double * detJ, double * K, std::size_t num_points,
      const double * x,
      const double * coordinate_dofs, int cell_orientation) const final override
  {
    const double phi_X0[3] = { 1.0, 0.0, 0.0 };
    const double dphi_X0[2][3] =
        { { -1.0, 1.0, 0.0 },
          { -1.0, 0.0, 1.0 } };
    double x0[2] = {};
    for (std::size_t i = 0; i < 2; ++i)
        for (std::size_t k = 0; k < 3; ++k)
            x0[i] += coordinate_dofs[2 * k + i] * phi_X0[k];
    for (std::size_t i = 0; i < 2; ++i)
    {
        for (std::size_t j = 0; j < 2; ++j)
        {
            J[2 * i + j] = 0.0;
            for (std::size_t k = 0; k < 3; ++k)
                J[2 * i + j] += coordinate_dofs[2 * k + i] * dphi_X0[j][k];
        }
    }
    compute_jacobian_determinants(detJ, 1, J, cell_orientation);
    compute_jacobian_inverses(K, 1, J, detJ);
    for (std::size_t ip = 0; ip < num_points; ++ip)
        for (std::size_t j = 0; j < 2; ++j)
            for (std::size_t i = 0; i < 2; ++i)
                X[2 * ip + j] += K[2 * j + i] * (x[2 * ip + i] - x0[i]);
  }

  void compute_jacobians(
      double * J, std::size_t num_points,
      const double * X,
      const double * coordinate_dofs) const final override
  {
    biharmonic_finite_element_0 xelement;
    double dphi[6];
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        // Compute basis derivatives of coordinate element
        xelement.evaluate_reference_basis_derivatives(dphi, 1, 1, &X[2 * ip]);
        // Compute J
        for (std::size_t i = 0; i < 2; ++i)
            for (std::size_t j = 0; j < 2; ++j)
                for (std::size_t d = 0; d < 3; ++d)
                    J[2 * 2 * ip + 2 * i + j] += coordinate_dofs[2 * d + i] * dphi[2 * d + j];
    }
  }

  void compute_jacobian_determinants(
      double * detJ, std::size_t num_points,
      const double * J,
      int cell_orientation) const final override
  {
    for (std::size_t ip = 0; ip < num_points; ++ip)
        detJ[ip] = J[2 * 2 * ip] * J[2 * 2 * ip + 2 + 1] - J[2 * 2 * ip + 1] * J[2 * 2 * ip + 2];
  }

  void compute_jacobian_inverses(
      double * K, std::size_t num_points,
      const double * J, const double * detJ) const final override
  {
    for (std::size_t ip = 0; ip < num_points; ++ip)
    {
        K[2 * 2 * ip] = J[2 * 2 * ip + 2 + 1] / detJ[ip];
        K[2 * 2 * ip + 1] = -J[2 * 2 * ip + 1] / detJ[ip];
        K[2 * 2 * ip + 2] = -J[2 * 2 * ip + 2] / detJ[ip];
        K[2 * 2 * ip + 2 + 1] = J[2 * 2 * ip] / detJ[ip];
    }
  }

  void compute_geometry(
      double * x, double * J, double * detJ, double * K, std::size_t num_points,
      const double * X,
      const double * coordinate_dofs, int cell_orientation) const final override
  {
    compute_physical_coordinates(x, num_points, X, coordinate_dofs);
    compute_jacobians(J, num_points, X, coordinate_dofs);
    compute_jacobian_determinants(detJ, num_points, J, cell_orientation);
    compute_jacobian_inverses(K, num_points, J, detJ);
  }

  void compute_midpoint_geometry(
      double * x, double * J,
      const double * coordinate_dofs) const final override
  {
    const double phi_Xm[3] = { 0.33333333333333337, 0.33333333333333337, 0.3333333333333333 };
    const double dphi_Xm[2][3] =
        { { -1.0, 1.0, 0.0 },
          { -1.0, 0.0, 1.0 } };
    // Compute x
    for (std::size_t i = 0; i < 2; ++i)
        for (std::size_t d = 0; d < 3; ++d)
            x[i] += coordinate_dofs[2 * d + i] * phi_Xm[d];
    // Compute J
    for (std::size_t i = 0; i < 2; ++i)
        for (std::size_t j = 0; j < 2; ++j)
            for (std::size_t d = 0; d < 3; ++d)
                J[2 * i + j] += coordinate_dofs[2 * d + i] * dphi_Xm[j][d];
  }

};


class biharmonic_cell_integral_0_otherwise: public ufc::cell_integral
{
public:

  biharmonic_cell_integral_0_otherwise() : ufc::cell_integral()
  {

  }

  ~biharmonic_cell_integral_0_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({false});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs,
                       int cell_orientation,
                       std::size_t local_facet) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    // num_cells:         None
    // optimize:          True
    // precision:         16
    // quadrature_degree: 0
    // quadrature_rule:   'default'
    // representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    // estimated_polynomial_degree: 0
    // optimize:                    True
    // precision:                   16
    // quadrature_degree:           0
    // quadrature_rule:             'default'
    // representation:              'uflacs'
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE3_C0_D01_Q1[1][1][2] = { { { -1.0, 1.0 } } };
    alignas(32) static const double PI0[1][3][3] =
        { { { 8.000000000000007, 8.000000000000005, -16.00000000000001 },
            { 8.000000000000005, 8.000000000000004, -16.0 },
            { -16.00000000000001, -16.0, 32.0 } } };
    alignas(32) static const double PI1[1][3][4] =
        { { { 8.000000000000009, 7.999999999999992, -7.999999999999992, -8.000000000000025 },
            { 8.000000000000007, 7.99999999999999, -7.99999999999999, -8.000000000000023 },
            { -16.00000000000001, -15.99999999999998, 15.99999999999998, 16.00000000000004 } } };
    alignas(32) static const double PI2[1][4][4] =
        { { { 8.000000000000011, 7.999999999999994, -7.999999999999994, -8.000000000000027 },
            { 7.999999999999994, 7.999999999999977, -7.999999999999977, -8.000000000000009 },
            { -7.999999999999994, -7.999999999999977, 7.999999999999977, 8.000000000000009 },
            { -8.000000000000027, -8.000000000000009, 8.000000000000009, 8.000000000000043 } } };
    // Unstructured piecewise computations
    const double J_c0 = coordinate_dofs[0] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[2] * FE3_C0_D01_Q1[0][0][1];
    const double J_c3 = coordinate_dofs[1] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[5] * FE3_C0_D01_Q1[0][0][1];
    const double J_c1 = coordinate_dofs[0] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[4] * FE3_C0_D01_Q1[0][0][1];
    const double J_c2 = coordinate_dofs[1] * FE3_C0_D01_Q1[0][0][0] + coordinate_dofs[3] * FE3_C0_D01_Q1[0][0][1];
    alignas(32) double sp[31];
    sp[0] = J_c0 * J_c3;
    sp[1] = J_c1 * J_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = J_c0 / sp[2];
    sp[4] = -1 * J_c1 / sp[2];
    sp[5] = sp[3] * sp[3];
    sp[6] = sp[3] * sp[4];
    sp[7] = sp[4] * sp[4];
    sp[8] = sp[6] + sp[6];
    sp[9] = J_c3 / sp[2];
    sp[10] = -1 * J_c2 / sp[2];
    sp[11] = sp[9] * sp[10];
    sp[12] = sp[9] * sp[9];
    sp[13] = sp[10] * sp[10];
    sp[14] = sp[11] + sp[11];
    sp[15] = sp[5] + sp[13];
    sp[16] = sp[8] + sp[14];
    sp[17] = sp[12] + sp[7];
    sp[18] = sp[15] * sp[15];
    sp[19] = sp[16] * sp[15];
    sp[20] = sp[17] * sp[15];
    sp[21] = sp[16] * sp[16];
    sp[22] = sp[16] * sp[17];
    sp[23] = sp[17] * sp[17];
    sp[24] = std::abs(sp[2]);
    sp[25] = sp[18] * sp[24];
    sp[26] = sp[19] * sp[24];
    sp[27] = sp[20] * sp[24];
    sp[28] = sp[21] * sp[24];
    sp[29] = sp[22] * sp[24];
    sp[30] = sp[23] * sp[24];
    A[0] = sp[30] * PI0[0][0][0] + sp[27] * PI0[0][0][0] + sp[29] * PI1[0][0][0] + sp[27] * PI0[0][0][0] + sp[25] * PI0[0][0][0] + sp[26] * PI1[0][0][0] + sp[29] * PI1[0][0][0] + sp[26] * PI1[0][0][0] + sp[28] * PI2[0][0][0];
    A[1] = sp[30] * PI0[0][0][1] + sp[27] * PI0[0][0][1] + sp[29] * PI1[0][1][0];
    A[2] = sp[27] * PI0[0][0][1] + sp[25] * PI0[0][0][1] + sp[26] * PI1[0][1][0];
    A[3] = sp[29] * PI1[0][0][1] + sp[26] * PI1[0][0][1] + sp[28] * PI2[0][0][1];
    A[4] = sp[27] * PI0[0][0][2] + sp[29] * PI1[0][0][2] + sp[25] * PI0[0][0][2] + sp[26] * PI1[0][0][2] + sp[26] * PI1[0][2][0] + sp[28] * PI2[0][0][2];
    A[5] = sp[30] * PI0[0][0][2] + sp[29] * PI1[0][0][3] + sp[27] * PI0[0][0][2] + sp[26] * PI1[0][0][3] + sp[29] * PI1[0][2][0] + sp[28] * PI2[0][0][3];
    A[6] = sp[30] * PI0[0][1][0] + sp[27] * PI0[0][1][0] + sp[29] * PI1[0][1][0];
    A[7] = sp[30] * PI0[0][1][1];
    A[8] = sp[27] * PI0[0][1][1];
    A[9] = sp[29] * PI1[0][1][1];
    A[10] = sp[27] * PI0[0][1][2] + sp[29] * PI1[0][1][2];
    A[11] = sp[30] * PI0[0][1][2] + sp[29] * PI1[0][1][3];
    A[12] = sp[27] * PI0[0][1][0] + sp[25] * PI0[0][1][0] + sp[26] * PI1[0][1][0];
    A[13] = sp[27] * PI0[0][1][1];
    A[14] = sp[25] * PI0[0][1][1];
    A[15] = sp[26] * PI1[0][1][1];
    A[16] = sp[25] * PI0[0][1][2] + sp[26] * PI1[0][1][2];
    A[17] = sp[27] * PI0[0][1][2] + sp[26] * PI1[0][1][3];
    A[18] = sp[29] * PI1[0][0][1] + sp[26] * PI1[0][0][1] + sp[28] * PI2[0][1][0];
    A[19] = sp[29] * PI1[0][1][1];
    A[20] = sp[26] * PI1[0][1][1];
    A[21] = sp[28] * PI2[0][1][1];
    A[22] = sp[26] * PI1[0][2][1] + sp[28] * PI2[0][1][2];
    A[23] = sp[29] * PI1[0][2][1] + sp[28] * PI2[0][1][3];
    A[24] = sp[27] * PI0[0][2][0] + sp[25] * PI0[0][2][0] + sp[26] * PI1[0][2][0] + sp[29] * PI1[0][0][2] + sp[26] * PI1[0][0][2] + sp[28] * PI2[0][2][0];
    A[25] = sp[27] * PI0[0][2][1] + sp[29] * PI1[0][1][2];
    A[26] = sp[25] * PI0[0][2][1] + sp[26] * PI1[0][1][2];
    A[27] = sp[26] * PI1[0][2][1] + sp[28] * PI2[0][2][1];
    A[28] = sp[25] * PI0[0][2][2] + sp[26] * PI1[0][2][2] + sp[26] * PI1[0][2][2] + sp[28] * PI2[0][2][2];
    A[29] = sp[27] * PI0[0][2][2] + sp[26] * PI1[0][2][3] + sp[29] * PI1[0][2][2] + sp[28] * PI2[0][2][3];
    A[30] = sp[30] * PI0[0][2][0] + sp[27] * PI0[0][2][0] + sp[29] * PI1[0][2][0] + sp[29] * PI1[0][0][3] + sp[26] * PI1[0][0][3] + sp[28] * PI2[0][3][0];
    A[31] = sp[30] * PI0[0][2][1] + sp[29] * PI1[0][1][3];
    A[32] = sp[27] * PI0[0][2][1] + sp[26] * PI1[0][1][3];
    A[33] = sp[29] * PI1[0][2][1] + sp[28] * PI2[0][3][1];
    A[34] = sp[27] * PI0[0][2][2] + sp[29] * PI1[0][2][2] + sp[26] * PI1[0][2][3] + sp[28] * PI2[0][3][2];
    A[35] = sp[30] * PI0[0][2][2] + sp[29] * PI1[0][2][3] + sp[29] * PI1[0][2][3] + sp[28] * PI2[0][3][3];
  }

};


class biharmonic_interior_facet_integral_0_otherwise: public ufc::interior_facet_integral
{
public:

  biharmonic_interior_facet_integral_0_otherwise() : ufc::interior_facet_integral()
  {

  }

  ~biharmonic_interior_facet_integral_0_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({true});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs_0,
                       const double * coordinate_dofs_1,
                       std::size_t facet_0,
                       std::size_t facet_1,
                       int cell_orientation_0,
                       int cell_orientation_1) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    // num_cells:         None
    // optimize:          True
    // precision:         16
    // quadrature_degree: 2
    // quadrature_rule:   'default'
    // representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    // estimated_polynomial_degree: 2
    // optimize:                    True
    // precision:                   16
    // quadrature_degree:           2
    // quadrature_rule:             'default'
    // representation:              'uflacs'
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE7_C0_D01_F_Q2[1][1][2] = { { { -1.0, 1.0 } } };
    alignas(32) static const double PI0[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999997, -1.999999999999997, -2.000000000000002 },
              { 1.0, 2.333333333333335, 0.666666666666665, -0.666666666666665, -3.333333333333339 },
              { 1.999999999999997, 0.666666666666665, 5.333333333333316, -5.333333333333316, -2.666666666666664 },
              { -1.999999999999997, -0.666666666666665, -5.333333333333316, 5.333333333333316, 2.666666666666664 },
              { -2.000000000000002, -3.333333333333339, -2.666666666666664, 2.666666666666664, 5.333333333333346 } },
            { { -1.0, -1.0, 1.999999999999997, -1.999999999999997, 2.000000000000003 },
              { -2.333333333333334, -1.0, 0.666666666666665, -0.666666666666665, 3.333333333333337 },
              { -0.6666666666666676, -1.999999999999997, 5.333333333333316, -5.333333333333316, 2.666666666666667 },
              { 0.6666666666666676, 1.999999999999997, -5.333333333333316, 5.333333333333316, -2.666666666666667 },
              { 3.333333333333339, 2.000000000000002, -2.666666666666664, 2.666666666666664, -5.333333333333345 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { -2.333333333333334, -0.3333333333333366, 0.0, 0.0, 2.666666666666671 },
              { -0.6666666666666615, 3.333333333333325, 0.0, 0.0, -2.666666666666664 },
              { 0.6666666666666615, -3.333333333333325, 0.0, 0.0, 2.666666666666664 },
              { 3.333333333333336, -0.6666666666666616, 0.0, 0.0, -2.666666666666675 } } },
          { { { -1.0, -2.333333333333334, -0.6666666666666676, 0.6666666666666676, 3.333333333333339 },
              { -1.0, -1.0, -1.999999999999997, 1.999999999999997, 2.000000000000002 },
              { 1.999999999999997, 0.666666666666665, 5.333333333333316, -5.333333333333316, -2.666666666666664 },
              { -1.999999999999997, -0.666666666666665, -5.333333333333316, 5.333333333333316, 2.666666666666664 },
              { 2.000000000000003, 3.333333333333337, 2.666666666666667, -2.666666666666667, -5.333333333333345 } },
            { { 2.333333333333334, 1.0, -0.6666666666666676, 0.6666666666666676, -3.333333333333337 },
              { 1.0, 1.0, -1.999999999999997, 1.999999999999997, -2.000000000000003 },
              { -0.6666666666666676, -1.999999999999997, 5.333333333333316, -5.333333333333316, 2.666666666666667 },
              { 0.6666666666666676, 1.999999999999997, -5.333333333333316, 5.333333333333316, -2.666666666666667 },
              { -3.333333333333337, -2.000000000000003, 2.666666666666667, -2.666666666666667, 5.333333333333344 } },
            { { 2.333333333333333, 0.3333333333333347, 0.0, 0.0, -2.666666666666668 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666615, 3.333333333333325, 0.0, 0.0, -2.666666666666664 },
              { 0.6666666666666615, -3.333333333333325, 0.0, 0.0, 2.666666666666664 },
              { -3.333333333333334, 0.6666666666666649, 0.0, 0.0, 2.666666666666669 } } },
          { { { -1.0, -2.333333333333334, -0.6666666666666615, 0.6666666666666615, 3.333333333333336 },
              { 1.0, -0.3333333333333366, 3.333333333333325, -3.333333333333325, -0.6666666666666616 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 2.666666666666671, -2.666666666666664, 2.666666666666664, -2.666666666666675 } },
            { { 2.333333333333333, 1.0, -0.6666666666666615, 0.6666666666666615, -3.333333333333334 },
              { 0.3333333333333347, -1.0, 3.333333333333325, -3.333333333333325, 0.6666666666666649 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666668, 0.0, -2.666666666666664, 2.666666666666664, 2.666666666666669 } },
            { { 2.333333333333333, 0.3333333333333385, 0.0, 0.0, -2.666666666666671 },
              { 0.3333333333333385, 2.333333333333332, 0.0, 0.0, -2.66666666666667 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666671, -2.66666666666667, 0.0, 0.0, 5.333333333333341 } } } };
    alignas(32) static const double PI1[3][3][5][3] =
        { { { { 4.0, 3.999999999999998, -7.999999999999997 },
              { 4.0, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000007, -8.000000000000004, 16.00000000000001 } },
            { { 4.0, 3.999999999999998, -7.999999999999997 },
              { 4.0, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000007, -8.000000000000004, 16.00000000000001 } },
            { { 4.0, 3.999999999999998, -7.999999999999997 },
              { 4.0, 3.999999999999998, -7.999999999999998 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { -8.000000000000007, -8.000000000000004, 16.00000000000001 } } },
          { { { -4.000000000000003, -4.000000000000001, 8.000000000000004 },
              { -4.0, -3.999999999999998, 7.999999999999997 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000011, 8.000000000000007, -16.00000000000002 } },
            { { -4.000000000000003, -4.000000000000001, 8.000000000000004 },
              { -4.0, -3.999999999999998, 7.999999999999997 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000011, 8.000000000000007, -16.00000000000002 } },
            { { -4.000000000000003, -4.000000000000001, 8.000000000000004 },
              { -4.0, -3.999999999999998, 7.999999999999997 },
              { 7.999999999999986, 7.999999999999982, -15.99999999999997 },
              { -7.999999999999986, -7.999999999999982, 15.99999999999997 },
              { 8.000000000000011, 8.000000000000007, -16.00000000000002 } } },
          { { { -3.999999999999994, -3.999999999999992, 7.999999999999986 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999982 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } },
            { { -3.999999999999994, -3.999999999999992, 7.999999999999986 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999982 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } },
            { { -3.999999999999994, -3.999999999999992, 7.999999999999986 },
              { 3.999999999999992, 3.99999999999999, -7.999999999999982 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0 } } } };
    alignas(32) static const double PI2[3][3][5][4] =
        { { { { 4.000000000000004, 3.999999999999993, -3.999999999999993, -4.000000000000016 },
              { 4.000000000000004, 3.999999999999994, -3.999999999999994, -4.000000000000017 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { -8.000000000000016, -7.999999999999995, 7.999999999999995, 8.000000000000041 } },
            { { 4.000000000000004, 3.999999999999993, -3.999999999999993, -4.000000000000016 },
              { 4.000000000000004, 3.999999999999994, -3.999999999999994, -4.000000000000017 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { -8.000000000000016, -7.999999999999995, 7.999999999999995, 8.000000000000041 } },
            { { 4.000000000000004, 3.999999999999993, -3.999999999999993, -4.000000000000016 },
              { 4.000000000000004, 3.999999999999994, -3.999999999999994, -4.000000000000017 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { -8.000000000000016, -7.999999999999995, 7.999999999999995, 8.000000000000041 } } },
          { { { -4.000000000000007, -3.999999999999997, 3.999999999999997, 4.00000000000002 },
              { -4.000000000000004, -3.999999999999993, 3.999999999999993, 4.000000000000016 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { 8.000000000000018, 7.999999999999997, -7.999999999999997, -8.000000000000043 } },
            { { -4.000000000000007, -3.999999999999997, 3.999999999999997, 4.00000000000002 },
              { -4.000000000000004, -3.999999999999993, 3.999999999999993, 4.000000000000016 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { 8.000000000000018, 7.999999999999997, -7.999999999999997, -8.000000000000043 } },
            { { -4.000000000000007, -3.999999999999997, 3.999999999999997, 4.00000000000002 },
              { -4.000000000000004, -3.999999999999993, 3.999999999999993, 4.000000000000016 },
              { 7.999999999999994, 7.999999999999973, -7.999999999999973, -8.000000000000018 },
              { -7.999999999999994, -7.999999999999973, 7.999999999999973, 8.000000000000018 },
              { 8.000000000000018, 7.999999999999997, -7.999999999999997, -8.000000000000043 } } },
          { { { -3.999999999999998, -3.999999999999988, 3.999999999999988, 4.000000000000011 },
              { 3.999999999999996, 3.999999999999986, -3.999999999999986, -4.000000000000008 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } },
            { { -3.999999999999998, -3.999999999999988, 3.999999999999988, 4.000000000000011 },
              { 3.999999999999996, 3.999999999999986, -3.999999999999986, -4.000000000000008 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } },
            { { -3.999999999999998, -3.999999999999988, 3.999999999999988, 4.000000000000011 },
              { 3.999999999999996, 3.999999999999986, -3.999999999999986, -4.000000000000008 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0 } } } };
    alignas(32) static const double PI3[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999984, -1.999999999999989, -2.0 },
              { 1.0, -0.3333333333333328, 3.333333333333319, -0.6666666666666544, -3.333333333333337 },
              { 1.999999999999997, 3.333333333333333, 2.666666666666629, -5.333333333333305, -2.666666666666661 },
              { -1.999999999999997, -3.333333333333333, -2.666666666666629, 5.333333333333305, 2.666666666666661 },
              { -2.000000000000002, -0.666666666666669, -5.333333333333309, 2.666666666666646, 5.333333333333343 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { -2.333333333333338, -0.3333333333333328, 0.0, 2.666666666666675, 0.0 },
              { -0.6666666666666756, 3.333333333333333, 0.0, -2.666666666666646, 0.0 },
              { 0.6666666666666756, -3.333333333333333, 0.0, 2.666666666666646, 0.0 },
              { 3.333333333333347, -0.666666666666669, 0.0, -2.666666666666687, 0.0 } },
            { { -1.0, -1.0, 1.999999999999988, 2.00000000000001, -2.0 },
              { -2.333333333333338, -1.0, 0.6666666666666559, 3.333333333333342, -0.6666666666666662 },
              { -0.6666666666666745, -1.999999999999997, 5.333333333333298, 2.666666666666684, -5.333333333333327 },
              { 0.6666666666666745, 1.999999999999997, -5.333333333333298, -2.666666666666684, 5.333333333333327 },
              { 3.333333333333346, 2.000000000000002, -2.666666666666646, -5.333333333333358, 2.666666666666669 } } },
          { { { -1.0, 0.3333333333333309, -3.333333333333319, 0.6666666666666571, 3.333333333333337 },
              { -1.0, -1.0, -1.999999999999984, 1.999999999999989, 2.0 },
              { 1.999999999999997, 3.333333333333333, 2.666666666666629, -5.333333333333305, -2.666666666666661 },
              { -1.999999999999997, -3.333333333333333, -2.666666666666629, 5.333333333333305, 2.666666666666661 },
              { 2.000000000000003, 0.6666666666666723, 5.333333333333307, -2.666666666666649, -5.33333333333334 } },
            { { 2.333333333333338, 0.3333333333333309, 0.0, -2.666666666666673, 0.0 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666756, 3.333333333333333, 0.0, -2.666666666666646, 0.0 },
              { 0.6666666666666756, -3.333333333333333, 0.0, 2.666666666666646, 0.0 },
              { -3.333333333333345, 0.6666666666666723, 0.0, 2.666666666666682, 0.0 } },
            { { 2.333333333333337, 1.0, -0.6666666666666584, -3.333333333333342, 0.6666666666666687 },
              { 1.0, 1.0, -1.999999999999988, -2.00000000000001, 2.0 },
              { -0.6666666666666745, -1.999999999999997, 5.333333333333298, 2.666666666666684, -5.333333333333327 },
              { 0.6666666666666745, 1.999999999999997, -5.333333333333298, -2.666666666666684, 5.333333333333327 },
              { -3.333333333333344, -2.000000000000003, 2.666666666666649, 5.333333333333357, -2.666666666666672 } } },
          { { { -1.0, 0.3333333333333347, -3.333333333333316, 0.666666666666651, 3.333333333333333 },
              { 1.0, 2.333333333333335, 0.6666666666666451, -3.333333333333321, -0.6666666666666606 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, -2.66666666666667, 2.666666666666671, 2.66666666666667, -2.666666666666673 } },
            { { 2.333333333333337, 0.3333333333333347, 0.0, -2.666666666666676, 0.0 },
              { 0.3333333333333307, 2.333333333333335, 0.0, -2.666666666666659, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666668, -2.66666666666667, 0.0, 5.333333333333336, 0.0 } },
            { { 2.333333333333337, 1.0, -0.6666666666666525, -3.333333333333339, 0.6666666666666629 },
              { 0.3333333333333313, -1.0, 3.333333333333316, 0.6666666666666741, -3.333333333333332 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666668, 0.0, -2.666666666666663, 2.666666666666665, 2.66666666666667 } } } };
    alignas(32) static const double PI4[3][3][3][5] =
        { { { { 4.0, 4.000000000000006, 7.999999999999934, -7.999999999999956, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999931, -7.999999999999952, -7.999999999999996 },
              { -7.999999999999997, -8.000000000000011, -15.99999999999986, 15.99999999999991, 16.0 } },
            { { -4.000000000000019, 4.000000000000006, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.0, 7.999999999999948, 8.000000000000039, -8.000000000000002 },
              { -4.000000000000015, -3.999999999999998, 7.999999999999945, 8.000000000000036, -7.999999999999998 },
              { 8.000000000000032, 7.999999999999997, -15.9999999999999, -16.00000000000007, 16.0 } } },
          { { { 4.0, 4.000000000000006, 7.999999999999934, -7.999999999999956, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999931, -7.999999999999952, -7.999999999999996 },
              { -7.999999999999997, -8.000000000000011, -15.99999999999986, 15.99999999999991, 16.0 } },
            { { -4.000000000000019, 4.000000000000006, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.0, 7.999999999999948, 8.000000000000039, -8.000000000000002 },
              { -4.000000000000015, -3.999999999999998, 7.999999999999945, 8.000000000000036, -7.999999999999998 },
              { 8.000000000000032, 7.999999999999997, -15.9999999999999, -16.00000000000007, 16.0 } } },
          { { { 4.0, 4.000000000000006, 7.999999999999934, -7.999999999999956, -8.0 },
              { 3.999999999999998, 4.000000000000004, 7.999999999999931, -7.999999999999952, -7.999999999999996 },
              { -7.999999999999997, -8.000000000000011, -15.99999999999986, 15.99999999999991, 16.0 } },
            { { -4.000000000000019, 4.000000000000006, 0.0, 0.0, 0.0 },
              { -4.000000000000017, 4.000000000000004, 0.0, 0.0, 0.0 },
              { 8.000000000000036, -8.000000000000011, 0.0, 0.0, 0.0 } },
            { { -4.000000000000017, -4.0, 7.999999999999948, 8.000000000000039, -8.000000000000002 },
              { -4.000000000000015, -3.999999999999998, 7.999999999999945, 8.000000000000036, -7.999999999999998 },
              { 8.000000000000032, 7.999999999999997, -15.9999999999999, -16.00000000000007, 16.0 } } } };
    alignas(32) static const double PI5[3][3][4][5] =
        { { { { 4.000000000000004, 4.000000000000011, 7.999999999999942, -7.999999999999964, -8.000000000000007 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999922, -7.999999999999943, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999922, 7.999999999999943, 7.999999999999988 },
              { -4.000000000000016, -4.000000000000023, -7.999999999999966, 7.999999999999988, 8.000000000000032 } },
            { { -4.000000000000023, 4.000000000000011, 0.0, 0.0, 0.0 },
              { -4.000000000000013, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000013, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000036, -4.000000000000023, 0.0, 0.0, 0.0 } },
            { { -4.00000000000002, -4.000000000000004, 7.999999999999956, 8.000000000000046, -8.000000000000009 },
              { -4.000000000000011, -3.999999999999993, 7.999999999999936, 8.000000000000027, -7.999999999999988 },
              { 4.000000000000011, 3.999999999999993, -7.999999999999936, -8.000000000000027, 7.999999999999988 },
              { 4.000000000000033, 4.000000000000016, -7.999999999999981, -8.000000000000071, 8.000000000000034 } } },
          { { { 4.000000000000004, 4.000000000000011, 7.999999999999942, -7.999999999999964, -8.000000000000007 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999922, -7.999999999999943, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999922, 7.999999999999943, 7.999999999999988 },
              { -4.000000000000016, -4.000000000000023, -7.999999999999966, 7.999999999999988, 8.000000000000032 } },
            { { -4.000000000000023, 4.000000000000011, 0.0, 0.0, 0.0 },
              { -4.000000000000013, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000013, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000036, -4.000000000000023, 0.0, 0.0, 0.0 } },
            { { -4.00000000000002, -4.000000000000004, 7.999999999999956, 8.000000000000046, -8.000000000000009 },
              { -4.000000000000011, -3.999999999999993, 7.999999999999936, 8.000000000000027, -7.999999999999988 },
              { 4.000000000000011, 3.999999999999993, -7.999999999999936, -8.000000000000027, 7.999999999999988 },
              { 4.000000000000033, 4.000000000000016, -7.999999999999981, -8.000000000000071, 8.000000000000034 } } },
          { { { 4.000000000000004, 4.000000000000011, 7.999999999999942, -7.999999999999964, -8.000000000000007 },
              { 3.999999999999993, 4.000000000000001, 7.999999999999922, -7.999999999999943, -7.999999999999988 },
              { -3.999999999999993, -4.000000000000001, -7.999999999999922, 7.999999999999943, 7.999999999999988 },
              { -4.000000000000016, -4.000000000000023, -7.999999999999966, 7.999999999999988, 8.000000000000032 } },
            { { -4.000000000000023, 4.000000000000011, 0.0, 0.0, 0.0 },
              { -4.000000000000013, 4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000013, -4.000000000000001, 0.0, 0.0, 0.0 },
              { 4.000000000000036, -4.000000000000023, 0.0, 0.0, 0.0 } },
            { { -4.00000000000002, -4.000000000000004, 7.999999999999956, 8.000000000000046, -8.000000000000009 },
              { -4.000000000000011, -3.999999999999993, 7.999999999999936, 8.000000000000027, -7.999999999999988 },
              { 4.000000000000011, 3.999999999999993, -7.999999999999936, -8.000000000000027, 7.999999999999988 },
              { 4.000000000000033, 4.000000000000016, -7.999999999999981, -8.000000000000071, 8.000000000000034 } } } };
    alignas(32) static const double PI6[3][3][5][5] =
        { { { { 1.0, 1.0, 1.999999999999984, -1.999999999999989, -2.0 },
              { 1.0, 2.333333333333339, 0.6666666666666525, -3.333333333333328, -0.6666666666666681 },
              { 1.999999999999984, 0.6666666666666525, 5.33333333333327, -2.666666666666611, -5.333333333333304 },
              { -1.999999999999989, -3.333333333333328, -2.666666666666611, 5.333333333333292, 2.666666666666643 },
              { -2.0, -0.6666666666666681, -5.333333333333304, 2.666666666666643, 5.333333333333337 } },
            { { -1.0, 1.0, 0.0, 0.0, 0.0 },
              { 0.3333333333333269, 2.333333333333339, 0.0, -2.666666666666659, 0.0 },
              { -3.333333333333327, 0.6666666666666525, 0.0, 2.666666666666684, 0.0 },
              { 0.6666666666666651, -3.333333333333328, 0.0, 2.666666666666651, 0.0 },
              { 3.333333333333344, -0.6666666666666681, 0.0, -2.666666666666686, 0.0 } },
            { { -1.0, -1.0, 1.999999999999988, 2.00000000000001, -2.0 },
              { 0.3333333333333275, -1.0, 3.333333333333323, 0.6666666666666816, -3.333333333333339 },
              { -3.333333333333326, -1.999999999999984, 2.666666666666611, 5.333333333333319, -2.666666666666634 },
              { 0.666666666666664, 1.999999999999989, -5.333333333333286, -2.666666666666666, 5.333333333333315 },
              { 3.333333333333344, 2.0, -2.666666666666643, -5.333333333333353, 2.666666666666666 } } },
          { { { -1.0, 0.3333333333333269, -3.333333333333327, 0.6666666666666651, 3.333333333333344 },
              { 1.0, 2.333333333333339, 0.6666666666666525, -3.333333333333328, -0.6666666666666681 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { 0.0, -2.666666666666659, 2.666666666666684, 2.666666666666651, -2.666666666666686 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } },
            { { 2.333333333333342, 0.3333333333333269, 0.0, -2.666666666666673, 0.0 },
              { 0.3333333333333269, 2.333333333333339, 0.0, -2.666666666666659, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666673, -2.666666666666659, 0.0, 5.333333333333329, 0.0 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } },
            { { 2.333333333333341, 1.0, -0.6666666666666665, -3.33333333333335, 0.6666666666666767 },
              { 0.3333333333333275, -1.0, 3.333333333333323, 0.6666666666666816, -3.333333333333339 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 },
              { -2.666666666666673, 0.0, -2.666666666666646, 2.666666666666678, 2.666666666666651 },
              { 0.0, 0.0, 0.0, 0.0, 0.0 } } },
          { { { -1.0, 0.3333333333333275, -3.333333333333326, 0.666666666666664, 3.333333333333344 },
              { -1.0, -1.0, -1.999999999999984, 1.999999999999989, 2.0 },
              { 1.999999999999988, 3.333333333333323, 2.666666666666611, -5.333333333333286, -2.666666666666643 },
              { 2.00000000000001, 0.6666666666666816, 5.333333333333319, -2.666666666666666, -5.333333333333353 },
              { -2.0, -3.333333333333339, -2.666666666666634, 5.333333333333315, 2.666666666666666 } },
            { { 2.333333333333341, 0.3333333333333275, 0.0, -2.666666666666673, 0.0 },
              { 1.0, -1.0, 0.0, 0.0, 0.0 },
              { -0.6666666666666665, 3.333333333333323, 0.0, -2.666666666666646, 0.0 },
              { -3.33333333333335, 0.6666666666666816, 0.0, 2.666666666666678, 0.0 },
              { 0.6666666666666767, -3.333333333333339, 0.0, 2.666666666666651, 0.0 } },
            { { 2.333333333333341, 1.0, -0.6666666666666654, -3.333333333333349, 0.6666666666666756 },
              { 1.0, 1.0, -1.999999999999988, -2.00000000000001, 2.0 },
              { -0.6666666666666654, -1.999999999999988, 5.333333333333279, 2.666666666666666, -5.333333333333308 },
              { -3.333333333333349, -2.00000000000001, 2.666666666666666, 5.333333333333369, -2.666666666666689 },
              { 0.6666666666666756, 2.0, -5.333333333333308, -2.666666666666689, 5.333333333333337 } } } };
    // Unstructured piecewise computations
    const double J_r0_c0 = coordinate_dofs_0[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[2] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c3 = coordinate_dofs_0[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[5] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c1 = coordinate_dofs_0[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[4] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r0_c2 = coordinate_dofs_0[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_0[3] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c0 = coordinate_dofs_1[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[2] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c3 = coordinate_dofs_1[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[5] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c1 = coordinate_dofs_1[0] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[4] * FE7_C0_D01_F_Q2[0][0][1];
    const double J_r1_c2 = coordinate_dofs_1[1] * FE7_C0_D01_F_Q2[0][0][0] + coordinate_dofs_1[3] * FE7_C0_D01_F_Q2[0][0][1];
    alignas(32) double sp[186];
    sp[0] = J_r0_c0 * J_r0_c3;
    sp[1] = J_r0_c1 * J_r0_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = J_r0_c0 / sp[2];
    sp[4] = -1 * J_r0_c1 / sp[2];
    sp[5] = triangle_reference_facet_normals[facet_0][1] * sp[3];
    sp[6] = triangle_reference_facet_normals[facet_0][0] * sp[4];
    sp[7] = sp[5] + sp[6];
    sp[8] = sp[7] * sp[7];
    sp[9] = J_r0_c3 / sp[2];
    sp[10] = triangle_reference_facet_normals[facet_0][0] * sp[9];
    sp[11] = -1 * J_r0_c2 / sp[2];
    sp[12] = triangle_reference_facet_normals[facet_0][1] * sp[11];
    sp[13] = sp[10] + sp[12];
    sp[14] = sp[13] * sp[13];
    sp[15] = sp[8] + sp[14];
    sp[16] = std::sqrt(sp[15]);
    sp[17] = sp[7] / sp[16];
    sp[18] = sp[3] * sp[17];
    sp[19] = sp[4] * sp[17];
    sp[20] = sp[13] / sp[16];
    sp[21] = sp[11] * sp[20];
    sp[22] = sp[9] * sp[20];
    sp[23] = sp[18] + sp[21];
    sp[24] = sp[19] + sp[22];
    sp[25] = J_r1_c0 * J_r1_c3;
    sp[26] = J_r1_c1 * J_r1_c2;
    sp[27] = sp[25] + -1 * sp[26];
    sp[28] = J_r1_c0 / sp[27];
    sp[29] = -1 * J_r1_c1 / sp[27];
    sp[30] = triangle_reference_facet_normals[facet_1][1] * sp[28];
    sp[31] = triangle_reference_facet_normals[facet_1][0] * sp[29];
    sp[32] = sp[30] + sp[31];
    sp[33] = sp[32] * sp[32];
    sp[34] = J_r1_c3 / sp[27];
    sp[35] = triangle_reference_facet_normals[facet_1][0] * sp[34];
    sp[36] = -1 * J_r1_c2 / sp[27];
    sp[37] = triangle_reference_facet_normals[facet_1][1] * sp[36];
    sp[38] = sp[35] + sp[37];
    sp[39] = sp[38] * sp[38];
    sp[40] = sp[33] + sp[39];
    sp[41] = std::sqrt(sp[40]);
    sp[42] = sp[32] / sp[41];
    sp[43] = sp[28] * sp[42];
    sp[44] = sp[29] * sp[42];
    sp[45] = sp[38] / sp[41];
    sp[46] = sp[36] * sp[45];
    sp[47] = sp[34] * sp[45];
    sp[48] = sp[43] + sp[46];
    sp[49] = sp[44] + sp[47];
    sp[50] = sp[3] * sp[3];
    sp[51] = sp[3] * sp[4];
    sp[52] = sp[4] * sp[4];
    sp[53] = sp[51] + sp[51];
    sp[54] = sp[9] * sp[11];
    sp[55] = sp[9] * sp[9];
    sp[56] = sp[11] * sp[11];
    sp[57] = sp[54] + sp[54];
    sp[58] = sp[50] + sp[56];
    sp[59] = sp[53] + sp[57];
    sp[60] = sp[55] + sp[52];
    sp[61] = sp[28] * sp[28];
    sp[62] = sp[28] * sp[29];
    sp[63] = sp[29] * sp[29];
    sp[64] = sp[62] + sp[62];
    sp[65] = sp[34] * sp[36];
    sp[66] = sp[34] * sp[34];
    sp[67] = sp[36] * sp[36];
    sp[68] = sp[65] + sp[65];
    sp[69] = sp[61] + sp[67];
    sp[70] = sp[64] + sp[68];
    sp[71] = sp[66] + sp[63];
    sp[72] = sp[23] * (0.5 * sp[58]);
    sp[73] = sp[23] * (0.5 * sp[59]);
    sp[74] = sp[23] * (0.5 * sp[60]);
    sp[75] = sp[23] * (0.5 * sp[69]);
    sp[76] = sp[23] * (0.5 * sp[70]);
    sp[77] = sp[23] * (0.5 * sp[71]);
    sp[78] = sp[24] * (0.5 * sp[58]);
    sp[79] = sp[24] * (0.5 * sp[59]);
    sp[80] = sp[24] * (0.5 * sp[60]);
    sp[81] = sp[24] * (0.5 * sp[69]);
    sp[82] = sp[24] * (0.5 * sp[70]);
    sp[83] = sp[24] * (0.5 * sp[71]);
    sp[84] = sp[48] * (0.5 * sp[58]);
    sp[85] = sp[48] * (0.5 * sp[59]);
    sp[86] = sp[48] * (0.5 * sp[60]);
    sp[87] = sp[48] * (0.5 * sp[69]);
    sp[88] = sp[48] * (0.5 * sp[70]);
    sp[89] = sp[48] * (0.5 * sp[71]);
    sp[90] = sp[49] * (0.5 * sp[58]);
    sp[91] = sp[49] * (0.5 * sp[59]);
    sp[92] = sp[49] * (0.5 * sp[60]);
    sp[93] = sp[49] * (0.5 * sp[69]);
    sp[94] = sp[49] * (0.5 * sp[70]);
    sp[95] = sp[49] * (0.5 * sp[71]);
    sp[96] = sp[23] * sp[23];
    sp[97] = sp[24] * sp[23];
    sp[98] = sp[23] * sp[48];
    sp[99] = sp[23] * sp[49];
    sp[100] = sp[24] * sp[24];
    sp[101] = sp[24] * sp[48];
    sp[102] = sp[24] * sp[49];
    sp[103] = sp[48] * sp[48];
    sp[104] = sp[49] * sp[48];
    sp[105] = sp[49] * sp[49];
    sp[106] = (coordinate_dofs_0[2] - coordinate_dofs_0[4]) * (coordinate_dofs_0[2] - coordinate_dofs_0[4]);
    sp[107] = (coordinate_dofs_0[3] - coordinate_dofs_0[5]) * (coordinate_dofs_0[3] - coordinate_dofs_0[5]);
    sp[108] = sp[106] + sp[107];
    sp[109] = (coordinate_dofs_0[0] - coordinate_dofs_0[4]) * (coordinate_dofs_0[0] - coordinate_dofs_0[4]);
    sp[110] = (coordinate_dofs_0[1] - coordinate_dofs_0[5]) * (coordinate_dofs_0[1] - coordinate_dofs_0[5]);
    sp[111] = sp[109] + sp[110];
    sp[112] = std::max(sp[108], sp[111]);
    sp[113] = (coordinate_dofs_0[0] - coordinate_dofs_0[2]) * (coordinate_dofs_0[0] - coordinate_dofs_0[2]);
    sp[114] = (coordinate_dofs_0[1] - coordinate_dofs_0[3]) * (coordinate_dofs_0[1] - coordinate_dofs_0[3]);
    sp[115] = sp[113] + sp[114];
    sp[116] = std::max(sp[112], sp[115]);
    sp[117] = std::sqrt(sp[116]);
    sp[118] = (coordinate_dofs_1[2] - coordinate_dofs_1[4]) * (coordinate_dofs_1[2] - coordinate_dofs_1[4]);
    sp[119] = (coordinate_dofs_1[3] - coordinate_dofs_1[5]) * (coordinate_dofs_1[3] - coordinate_dofs_1[5]);
    sp[120] = sp[118] + sp[119];
    sp[121] = (coordinate_dofs_1[0] - coordinate_dofs_1[4]) * (coordinate_dofs_1[0] - coordinate_dofs_1[4]);
    sp[122] = (coordinate_dofs_1[1] - coordinate_dofs_1[5]) * (coordinate_dofs_1[1] - coordinate_dofs_1[5]);
    sp[123] = sp[121] + sp[122];
    sp[124] = std::max(sp[120], sp[123]);
    sp[125] = (coordinate_dofs_1[0] - coordinate_dofs_1[2]) * (coordinate_dofs_1[0] - coordinate_dofs_1[2]);
    sp[126] = (coordinate_dofs_1[1] - coordinate_dofs_1[3]) * (coordinate_dofs_1[1] - coordinate_dofs_1[3]);
    sp[127] = sp[125] + sp[126];
    sp[128] = std::max(sp[124], sp[127]);
    sp[129] = std::sqrt(sp[128]);
    sp[130] = sp[117] + sp[129];
    sp[131] = w[0][0] / (sp[130] / 2);
    sp[132] = sp[96] * sp[131];
    sp[133] = sp[97] * sp[131];
    sp[134] = sp[98] * sp[131];
    sp[135] = sp[99] * sp[131];
    sp[136] = sp[100] * sp[131];
    sp[137] = sp[101] * sp[131];
    sp[138] = sp[102] * sp[131];
    sp[139] = sp[103] * sp[131];
    sp[140] = sp[104] * sp[131];
    sp[141] = sp[105] * sp[131];
    sp[142] = J_r0_c0 * triangle_reference_facet_jacobian[facet_0][0][0];
    sp[143] = J_r0_c1 * triangle_reference_facet_jacobian[facet_0][1][0];
    sp[144] = sp[142] + sp[143];
    sp[145] = sp[144] * sp[144];
    sp[146] = triangle_reference_facet_jacobian[facet_0][0][0] * J_r0_c2;
    sp[147] = triangle_reference_facet_jacobian[facet_0][1][0] * J_r0_c3;
    sp[148] = sp[146] + sp[147];
    sp[149] = sp[148] * sp[148];
    sp[150] = sp[145] + sp[149];
    sp[151] = std::sqrt(sp[150]);
    sp[152] = -1 * sp[72] * sp[151];
    sp[153] = -1 * sp[73] * sp[151];
    sp[154] = -1 * sp[74] * sp[151];
    sp[155] = -1 * sp[75] * sp[151];
    sp[156] = -1 * sp[76] * sp[151];
    sp[157] = -1 * sp[77] * sp[151];
    sp[158] = sp[132] * sp[151];
    sp[159] = sp[133] * sp[151];
    sp[160] = sp[134] * sp[151];
    sp[161] = sp[135] * sp[151];
    sp[162] = -1 * sp[78] * sp[151];
    sp[163] = -1 * sp[79] * sp[151];
    sp[164] = -1 * sp[80] * sp[151];
    sp[165] = -1 * sp[81] * sp[151];
    sp[166] = -1 * sp[82] * sp[151];
    sp[167] = -1 * sp[83] * sp[151];
    sp[168] = sp[136] * sp[151];
    sp[169] = sp[137] * sp[151];
    sp[170] = sp[138] * sp[151];
    sp[171] = -1 * sp[84] * sp[151];
    sp[172] = -1 * sp[85] * sp[151];
    sp[173] = -1 * sp[86] * sp[151];
    sp[174] = -1 * sp[87] * sp[151];
    sp[175] = -1 * sp[88] * sp[151];
    sp[176] = -1 * sp[89] * sp[151];
    sp[177] = sp[139] * sp[151];
    sp[178] = sp[140] * sp[151];
    sp[179] = -1 * sp[90] * sp[151];
    sp[180] = -1 * sp[91] * sp[151];
    sp[181] = -1 * sp[92] * sp[151];
    sp[182] = -1 * sp[93] * sp[151];
    sp[183] = -1 * sp[94] * sp[151];
    sp[184] = -1 * sp[95] * sp[151];
    sp[185] = sp[141] * sp[151];
    A[0] = sp[168] * PI0[facet_0][facet_0][0][0] + sp[164] * PI1[facet_0][0][0][0] + sp[159] * PI3[facet_0][facet_0][0][0] + sp[162] * PI1[facet_0][0][0][0] + sp[163] * PI2[facet_0][0][0][0] + sp[164] * PI1[facet_0][0][0][0] + sp[154] * PI4[0][facet_0][0][0] + sp[159] * PI3[facet_0][facet_0][0][0] + sp[154] * PI4[0][facet_0][0][0] + sp[158] * PI6[facet_0][facet_0][0][0] + sp[152] * PI4[0][facet_0][0][0] + sp[153] * PI5[0][facet_0][0][0] + sp[162] * PI1[facet_0][0][0][0] + sp[152] * PI4[0][facet_0][0][0] + sp[163] * PI2[facet_0][0][0][0] + sp[153] * PI5[0][facet_0][0][0];
    A[1] = sp[168] * PI0[facet_0][facet_0][0][1] + sp[164] * PI1[facet_0][0][0][1] + sp[164] * PI1[facet_0][0][1][0] + sp[159] * PI3[facet_0][facet_0][1][0] + sp[154] * PI4[0][facet_0][1][0] + sp[162] * PI1[facet_0][0][1][0] + sp[163] * PI2[facet_0][0][1][0];
    A[2] = sp[159] * PI3[facet_0][facet_0][0][1] + sp[162] * PI1[facet_0][0][0][1] + sp[154] * PI4[0][facet_0][0][1] + sp[158] * PI6[facet_0][facet_0][0][1] + sp[152] * PI4[0][facet_0][1][0] + sp[152] * PI4[0][facet_0][0][1] + sp[153] * PI5[0][facet_0][0][1];
    A[3] = sp[168] * PI0[facet_0][facet_0][0][2] + sp[159] * PI3[facet_0][facet_0][0][2] + sp[163] * PI2[facet_0][0][0][1] + sp[164] * PI1[facet_0][0][2][0] + sp[154] * PI4[0][facet_0][0][2] + sp[159] * PI3[facet_0][facet_0][2][0] + sp[158] * PI6[facet_0][facet_0][0][2] + sp[153] * PI5[0][facet_0][1][0] + sp[162] * PI1[facet_0][0][2][0] + sp[152] * PI4[0][facet_0][0][2] + sp[163] * PI2[facet_0][0][2][0] + sp[153] * PI5[0][facet_0][0][2];
    A[4] = sp[168] * PI0[facet_0][facet_0][0][3] + sp[159] * PI3[facet_0][facet_0][0][3] + sp[162] * PI1[facet_0][0][0][2] + sp[163] * PI2[facet_0][0][0][2] + sp[164] * PI1[facet_0][0][3][0] + sp[154] * PI4[0][facet_0][0][3] + sp[159] * PI3[facet_0][facet_0][3][0] + sp[158] * PI6[facet_0][facet_0][0][3] + sp[152] * PI4[0][facet_0][2][0] + sp[153] * PI5[0][facet_0][2][0] + sp[162] * PI1[facet_0][0][3][0] + sp[152] * PI4[0][facet_0][0][3] + sp[163] * PI2[facet_0][0][3][0] + sp[153] * PI5[0][facet_0][0][3];
    A[5] = sp[168] * PI0[facet_0][facet_0][0][4] + sp[164] * PI1[facet_0][0][0][2] + sp[159] * PI3[facet_0][facet_0][0][4] + sp[163] * PI2[facet_0][0][0][3] + sp[164] * PI1[facet_0][0][4][0] + sp[154] * PI4[0][facet_0][0][4] + sp[159] * PI3[facet_0][facet_0][4][0] + sp[154] * PI4[0][facet_0][2][0] + sp[158] * PI6[facet_0][facet_0][0][4] + sp[153] * PI5[0][facet_0][3][0] + sp[162] * PI1[facet_0][0][4][0] + sp[152] * PI4[0][facet_0][0][4] + sp[163] * PI2[facet_0][0][4][0] + sp[153] * PI5[0][facet_0][0][4];
    A[6] = sp[170] * PI0[facet_0][facet_1][0][0] + sp[167] * PI1[facet_0][0][0][0] + sp[169] * PI3[facet_0][facet_1][0][0] + sp[165] * PI1[facet_0][0][0][0] + sp[166] * PI2[facet_0][0][0][0] + sp[181] * PI1[facet_1][0][0][0] + sp[173] * PI4[0][facet_1][0][0] + sp[161] * PI3[facet_1][facet_0][0][0] + sp[157] * PI4[0][facet_0][0][0] + sp[160] * PI6[facet_0][facet_1][0][0] + sp[155] * PI4[0][facet_0][0][0] + sp[156] * PI5[0][facet_0][0][0] + sp[179] * PI1[facet_1][0][0][0] + sp[171] * PI4[0][facet_1][0][0] + sp[180] * PI2[facet_1][0][0][0] + sp[172] * PI5[0][facet_1][0][0];
    A[7] = sp[170] * PI0[facet_0][facet_1][0][1] + sp[167] * PI1[facet_0][0][0][1] + sp[181] * PI1[facet_1][0][1][0] + sp[161] * PI3[facet_1][facet_0][1][0] + sp[157] * PI4[0][facet_0][1][0] + sp[179] * PI1[facet_1][0][1][0] + sp[180] * PI2[facet_1][0][1][0];
    A[8] = sp[169] * PI3[facet_0][facet_1][0][1] + sp[165] * PI1[facet_0][0][0][1] + sp[173] * PI4[0][facet_1][0][1] + sp[160] * PI6[facet_0][facet_1][0][1] + sp[155] * PI4[0][facet_0][1][0] + sp[171] * PI4[0][facet_1][0][1] + sp[172] * PI5[0][facet_1][0][1];
    A[9] = sp[170] * PI0[facet_0][facet_1][0][2] + sp[169] * PI3[facet_0][facet_1][0][2] + sp[166] * PI2[facet_0][0][0][1] + sp[181] * PI1[facet_1][0][2][0] + sp[173] * PI4[0][facet_1][0][2] + sp[161] * PI3[facet_1][facet_0][2][0] + sp[160] * PI6[facet_0][facet_1][0][2] + sp[156] * PI5[0][facet_0][1][0] + sp[179] * PI1[facet_1][0][2][0] + sp[171] * PI4[0][facet_1][0][2] + sp[180] * PI2[facet_1][0][2][0] + sp[172] * PI5[0][facet_1][0][2];
    A[10] = sp[170] * PI0[facet_0][facet_1][0][3] + sp[169] * PI3[facet_0][facet_1][0][3] + sp[165] * PI1[facet_0][0][0][2] + sp[166] * PI2[facet_0][0][0][2] + sp[181] * PI1[facet_1][0][3][0] + sp[173] * PI4[0][facet_1][0][3] + sp[161] * PI3[facet_1][facet_0][3][0] + sp[160] * PI6[facet_0][facet_1][0][3] + sp[155] * PI4[0][facet_0][2][0] + sp[156] * PI5[0][facet_0][2][0] + sp[179] * PI1[facet_1][0][3][0] + sp[171] * PI4[0][facet_1][0][3] + sp[180] * PI2[facet_1][0][3][0] + sp[172] * PI5[0][facet_1][0][3];
    A[11] = sp[170] * PI0[facet_0][facet_1][0][4] + sp[167] * PI1[facet_0][0][0][2] + sp[169] * PI3[facet_0][facet_1][0][4] + sp[166] * PI2[facet_0][0][0][3] + sp[181] * PI1[facet_1][0][4][0] + sp[173] * PI4[0][facet_1][0][4] + sp[161] * PI3[facet_1][facet_0][4][0] + sp[157] * PI4[0][facet_0][2][0] + sp[160] * PI6[facet_0][facet_1][0][4] + sp[156] * PI5[0][facet_0][3][0] + sp[179] * PI1[facet_1][0][4][0] + sp[171] * PI4[0][facet_1][0][4] + sp[180] * PI2[facet_1][0][4][0] + sp[172] * PI5[0][facet_1][0][4];
    A[12] = sp[168] * PI0[facet_0][facet_0][1][0] + sp[164] * PI1[facet_0][0][1][0] + sp[159] * PI3[facet_0][facet_0][1][0] + sp[162] * PI1[facet_0][0][1][0] + sp[163] * PI2[facet_0][0][1][0] + sp[164] * PI1[facet_0][0][0][1] + sp[154] * PI4[0][facet_0][1][0];
    A[13] = sp[168] * PI0[facet_0][facet_0][1][1] + sp[164] * PI1[facet_0][0][1][1] + sp[164] * PI1[facet_0][0][1][1];
    A[14] = sp[159] * PI3[facet_0][facet_0][1][1] + sp[162] * PI1[facet_0][0][1][1] + sp[154] * PI4[0][facet_0][1][1];
    A[15] = sp[168] * PI0[facet_0][facet_0][1][2] + sp[159] * PI3[facet_0][facet_0][1][2] + sp[163] * PI2[facet_0][0][1][1] + sp[164] * PI1[facet_0][0][2][1] + sp[154] * PI4[0][facet_0][1][2];
    A[16] = sp[168] * PI0[facet_0][facet_0][1][3] + sp[159] * PI3[facet_0][facet_0][1][3] + sp[162] * PI1[facet_0][0][1][2] + sp[163] * PI2[facet_0][0][1][2] + sp[164] * PI1[facet_0][0][3][1] + sp[154] * PI4[0][facet_0][1][3];
    A[17] = sp[168] * PI0[facet_0][facet_0][1][4] + sp[164] * PI1[facet_0][0][1][2] + sp[159] * PI3[facet_0][facet_0][1][4] + sp[163] * PI2[facet_0][0][1][3] + sp[164] * PI1[facet_0][0][4][1] + sp[154] * PI4[0][facet_0][1][4];
    A[18] = sp[170] * PI0[facet_0][facet_1][1][0] + sp[167] * PI1[facet_0][0][1][0] + sp[169] * PI3[facet_0][facet_1][1][0] + sp[165] * PI1[facet_0][0][1][0] + sp[166] * PI2[facet_0][0][1][0] + sp[181] * PI1[facet_1][0][0][1] + sp[173] * PI4[0][facet_1][1][0];
    A[19] = sp[170] * PI0[facet_0][facet_1][1][1] + sp[167] * PI1[facet_0][0][1][1] + sp[181] * PI1[facet_1][0][1][1];
    A[20] = sp[169] * PI3[facet_0][facet_1][1][1] + sp[165] * PI1[facet_0][0][1][1] + sp[173] * PI4[0][facet_1][1][1];
    A[21] = sp[170] * PI0[facet_0][facet_1][1][2] + sp[169] * PI3[facet_0][facet_1][1][2] + sp[166] * PI2[facet_0][0][1][1] + sp[181] * PI1[facet_1][0][2][1] + sp[173] * PI4[0][facet_1][1][2];
    A[22] = sp[170] * PI0[facet_0][facet_1][1][3] + sp[169] * PI3[facet_0][facet_1][1][3] + sp[165] * PI1[facet_0][0][1][2] + sp[166] * PI2[facet_0][0][1][2] + sp[181] * PI1[facet_1][0][3][1] + sp[173] * PI4[0][facet_1][1][3];
    A[23] = sp[170] * PI0[facet_0][facet_1][1][4] + sp[167] * PI1[facet_0][0][1][2] + sp[169] * PI3[facet_0][facet_1][1][4] + sp[166] * PI2[facet_0][0][1][3] + sp[181] * PI1[facet_1][0][4][1] + sp[173] * PI4[0][facet_1][1][4];
    A[24] = sp[159] * PI3[facet_0][facet_0][0][1] + sp[154] * PI4[0][facet_0][0][1] + sp[158] * PI6[facet_0][facet_0][1][0] + sp[152] * PI4[0][facet_0][0][1] + sp[153] * PI5[0][facet_0][0][1] + sp[162] * PI1[facet_0][0][0][1] + sp[152] * PI4[0][facet_0][1][0];
    A[25] = sp[159] * PI3[facet_0][facet_0][1][1] + sp[154] * PI4[0][facet_0][1][1] + sp[162] * PI1[facet_0][0][1][1];
    A[26] = sp[158] * PI6[facet_0][facet_0][1][1] + sp[152] * PI4[0][facet_0][1][1] + sp[152] * PI4[0][facet_0][1][1];
    A[27] = sp[159] * PI3[facet_0][facet_0][2][1] + sp[158] * PI6[facet_0][facet_0][1][2] + sp[153] * PI5[0][facet_0][1][1] + sp[162] * PI1[facet_0][0][2][1] + sp[152] * PI4[0][facet_0][1][2];
    A[28] = sp[159] * PI3[facet_0][facet_0][3][1] + sp[158] * PI6[facet_0][facet_0][1][3] + sp[152] * PI4[0][facet_0][2][1] + sp[153] * PI5[0][facet_0][2][1] + sp[162] * PI1[facet_0][0][3][1] + sp[152] * PI4[0][facet_0][1][3];
    A[29] = sp[159] * PI3[facet_0][facet_0][4][1] + sp[154] * PI4[0][facet_0][2][1] + sp[158] * PI6[facet_0][facet_0][1][4] + sp[153] * PI5[0][facet_0][3][1] + sp[162] * PI1[facet_0][0][4][1] + sp[152] * PI4[0][facet_0][1][4];
    A[30] = sp[161] * PI3[facet_1][facet_0][0][1] + sp[157] * PI4[0][facet_0][0][1] + sp[160] * PI6[facet_0][facet_1][1][0] + sp[155] * PI4[0][facet_0][0][1] + sp[156] * PI5[0][facet_0][0][1] + sp[179] * PI1[facet_1][0][0][1] + sp[171] * PI4[0][facet_1][1][0];
    A[31] = sp[161] * PI3[facet_1][facet_0][1][1] + sp[157] * PI4[0][facet_0][1][1] + sp[179] * PI1[facet_1][0][1][1];
    A[32] = sp[160] * PI6[facet_0][facet_1][1][1] + sp[155] * PI4[0][facet_0][1][1] + sp[171] * PI4[0][facet_1][1][1];
    A[33] = sp[161] * PI3[facet_1][facet_0][2][1] + sp[160] * PI6[facet_0][facet_1][1][2] + sp[156] * PI5[0][facet_0][1][1] + sp[179] * PI1[facet_1][0][2][1] + sp[171] * PI4[0][facet_1][1][2];
    A[34] = sp[161] * PI3[facet_1][facet_0][3][1] + sp[160] * PI6[facet_0][facet_1][1][3] + sp[155] * PI4[0][facet_0][2][1] + sp[156] * PI5[0][facet_0][2][1] + sp[179] * PI1[facet_1][0][3][1] + sp[171] * PI4[0][facet_1][1][3];
    A[35] = sp[161] * PI3[facet_1][facet_0][4][1] + sp[157] * PI4[0][facet_0][2][1] + sp[160] * PI6[facet_0][facet_1][1][4] + sp[156] * PI5[0][facet_0][3][1] + sp[179] * PI1[facet_1][0][4][1] + sp[171] * PI4[0][facet_1][1][4];
    A[36] = sp[168] * PI0[facet_0][facet_0][2][0] + sp[164] * PI1[facet_0][0][2][0] + sp[159] * PI3[facet_0][facet_0][2][0] + sp[162] * PI1[facet_0][0][2][0] + sp[163] * PI2[facet_0][0][2][0] + sp[159] * PI3[facet_0][facet_0][0][2] + sp[154] * PI4[0][facet_0][0][2] + sp[158] * PI6[facet_0][facet_0][2][0] + sp[152] * PI4[0][facet_0][0][2] + sp[153] * PI5[0][facet_0][0][2] + sp[163] * PI2[facet_0][0][0][1] + sp[153] * PI5[0][facet_0][1][0];
    A[37] = sp[168] * PI0[facet_0][facet_0][2][1] + sp[164] * PI1[facet_0][0][2][1] + sp[159] * PI3[facet_0][facet_0][1][2] + sp[154] * PI4[0][facet_0][1][2] + sp[163] * PI2[facet_0][0][1][1];
    A[38] = sp[159] * PI3[facet_0][facet_0][2][1] + sp[162] * PI1[facet_0][0][2][1] + sp[158] * PI6[facet_0][facet_0][2][1] + sp[152] * PI4[0][facet_0][1][2] + sp[153] * PI5[0][facet_0][1][1];
    A[39] = sp[168] * PI0[facet_0][facet_0][2][2] + sp[159] * PI3[facet_0][facet_0][2][2] + sp[163] * PI2[facet_0][0][2][1] + sp[159] * PI3[facet_0][facet_0][2][2] + sp[158] * PI6[facet_0][facet_0][2][2] + sp[153] * PI5[0][facet_0][1][2] + sp[163] * PI2[facet_0][0][2][1] + sp[153] * PI5[0][facet_0][1][2];
    A[40] = sp[168] * PI0[facet_0][facet_0][2][3] + sp[159] * PI3[facet_0][facet_0][2][3] + sp[162] * PI1[facet_0][0][2][2] + sp[163] * PI2[facet_0][0][2][2] + sp[159] * PI3[facet_0][facet_0][3][2] + sp[158] * PI6[facet_0][facet_0][2][3] + sp[152] * PI4[0][facet_0][2][2] + sp[153] * PI5[0][facet_0][2][2] + sp[163] * PI2[facet_0][0][3][1] + sp[153] * PI5[0][facet_0][1][3];
    A[41] = sp[168] * PI0[facet_0][facet_0][2][4] + sp[164] * PI1[facet_0][0][2][2] + sp[159] * PI3[facet_0][facet_0][2][4] + sp[163] * PI2[facet_0][0][2][3] + sp[159] * PI3[facet_0][facet_0][4][2] + sp[154] * PI4[0][facet_0][2][2] + sp[158] * PI6[facet_0][facet_0][2][4] + sp[153] * PI5[0][facet_0][3][2] + sp[163] * PI2[facet_0][0][4][1] + sp[153] * PI5[0][facet_0][1][4];
    A[42] = sp[170] * PI0[facet_0][facet_1][2][0] + sp[167] * PI1[facet_0][0][2][0] + sp[169] * PI3[facet_0][facet_1][2][0] + sp[165] * PI1[facet_0][0][2][0] + sp[166] * PI2[facet_0][0][2][0] + sp[161] * PI3[facet_1][facet_0][0][2] + sp[157] * PI4[0][facet_0][0][2] + sp[160] * PI6[facet_0][facet_1][2][0] + sp[155] * PI4[0][facet_0][0][2] + sp[156] * PI5[0][facet_0][0][2] + sp[180] * PI2[facet_1][0][0][1] + sp[172] * PI5[0][facet_1][1][0];
    A[43] = sp[170] * PI0[facet_0][facet_1][2][1] + sp[167] * PI1[facet_0][0][2][1] + sp[161] * PI3[facet_1][facet_0][1][2] + sp[157] * PI4[0][facet_0][1][2] + sp[180] * PI2[facet_1][0][1][1];
    A[44] = sp[169] * PI3[facet_0][facet_1][2][1] + sp[165] * PI1[facet_0][0][2][1] + sp[160] * PI6[facet_0][facet_1][2][1] + sp[155] * PI4[0][facet_0][1][2] + sp[172] * PI5[0][facet_1][1][1];
    A[45] = sp[170] * PI0[facet_0][facet_1][2][2] + sp[169] * PI3[facet_0][facet_1][2][2] + sp[166] * PI2[facet_0][0][2][1] + sp[161] * PI3[facet_1][facet_0][2][2] + sp[160] * PI6[facet_0][facet_1][2][2] + sp[156] * PI5[0][facet_0][1][2] + sp[180] * PI2[facet_1][0][2][1] + sp[172] * PI5[0][facet_1][1][2];
    A[46] = sp[170] * PI0[facet_0][facet_1][2][3] + sp[169] * PI3[facet_0][facet_1][2][3] + sp[165] * PI1[facet_0][0][2][2] + sp[166] * PI2[facet_0][0][2][2] + sp[161] * PI3[facet_1][facet_0][3][2] + sp[160] * PI6[facet_0][facet_1][2][3] + sp[155] * PI4[0][facet_0][2][2] + sp[156] * PI5[0][facet_0][2][2] + sp[180] * PI2[facet_1][0][3][1] + sp[172] * PI5[0][facet_1][1][3];
    A[47] = sp[170] * PI0[facet_0][facet_1][2][4] + sp[167] * PI1[facet_0][0][2][2] + sp[169] * PI3[facet_0][facet_1][2][4] + sp[166] * PI2[facet_0][0][2][3] + sp[161] * PI3[facet_1][facet_0][4][2] + sp[157] * PI4[0][facet_0][2][2] + sp[160] * PI6[facet_0][facet_1][2][4] + sp[156] * PI5[0][facet_0][3][2] + sp[180] * PI2[facet_1][0][4][1] + sp[172] * PI5[0][facet_1][1][4];
    A[48] = sp[168] * PI0[facet_0][facet_0][3][0] + sp[164] * PI1[facet_0][0][3][0] + sp[159] * PI3[facet_0][facet_0][3][0] + sp[162] * PI1[facet_0][0][3][0] + sp[163] * PI2[facet_0][0][3][0] + sp[159] * PI3[facet_0][facet_0][0][3] + sp[154] * PI4[0][facet_0][0][3] + sp[158] * PI6[facet_0][facet_0][3][0] + sp[152] * PI4[0][facet_0][0][3] + sp[153] * PI5[0][facet_0][0][3] + sp[162] * PI1[facet_0][0][0][2] + sp[152] * PI4[0][facet_0][2][0] + sp[163] * PI2[facet_0][0][0][2] + sp[153] * PI5[0][facet_0][2][0];
    A[49] = sp[168] * PI0[facet_0][facet_0][3][1] + sp[164] * PI1[facet_0][0][3][1] + sp[159] * PI3[facet_0][facet_0][1][3] + sp[154] * PI4[0][facet_0][1][3] + sp[162] * PI1[facet_0][0][1][2] + sp[163] * PI2[facet_0][0][1][2];
    A[50] = sp[159] * PI3[facet_0][facet_0][3][1] + sp[162] * PI1[facet_0][0][3][1] + sp[158] * PI6[facet_0][facet_0][3][1] + sp[152] * PI4[0][facet_0][1][3] + sp[152] * PI4[0][facet_0][2][1] + sp[153] * PI5[0][facet_0][2][1];
    A[51] = sp[168] * PI0[facet_0][facet_0][3][2] + sp[159] * PI3[facet_0][facet_0][3][2] + sp[163] * PI2[facet_0][0][3][1] + sp[159] * PI3[facet_0][facet_0][2][3] + sp[158] * PI6[facet_0][facet_0][3][2] + sp[153] * PI5[0][facet_0][1][3] + sp[162] * PI1[facet_0][0][2][2] + sp[152] * PI4[0][facet_0][2][2] + sp[163] * PI2[facet_0][0][2][2] + sp[153] * PI5[0][facet_0][2][2];
    A[52] = sp[168] * PI0[facet_0][facet_0][3][3] + sp[159] * PI3[facet_0][facet_0][3][3] + sp[162] * PI1[facet_0][0][3][2] + sp[163] * PI2[facet_0][0][3][2] + sp[159] * PI3[facet_0][facet_0][3][3] + sp[158] * PI6[facet_0][facet_0][3][3] + sp[152] * PI4[0][facet_0][2][3] + sp[153] * PI5[0][facet_0][2][3] + sp[162] * PI1[facet_0][0][3][2] + sp[152] * PI4[0][facet_0][2][3] + sp[163] * PI2[facet_0][0][3][2] + sp[153] * PI5[0][facet_0][2][3];
    A[53] = sp[168] * PI0[facet_0][facet_0][3][4] + sp[164] * PI1[facet_0][0][3][2] + sp[159] * PI3[facet_0][facet_0][3][4] + sp[163] * PI2[facet_0][0][3][3] + sp[159] * PI3[facet_0][facet_0][4][3] + sp[154] * PI4[0][facet_0][2][3] + sp[158] * PI6[facet_0][facet_0][3][4] + sp[153] * PI5[0][facet_0][3][3] + sp[162] * PI1[facet_0][0][4][2] + sp[152] * PI4[0][facet_0][2][4] + sp[163] * PI2[facet_0][0][4][2] + sp[153] * PI5[0][facet_0][2][4];
    A[54] = sp[170] * PI0[facet_0][facet_1][3][0] + sp[167] * PI1[facet_0][0][3][0] + sp[169] * PI3[facet_0][facet_1][3][0] + sp[165] * PI1[facet_0][0][3][0] + sp[166] * PI2[facet_0][0][3][0] + sp[161] * PI3[facet_1][facet_0][0][3] + sp[157] * PI4[0][facet_0][0][3] + sp[160] * PI6[facet_0][facet_1][3][0] + sp[155] * PI4[0][facet_0][0][3] + sp[156] * PI5[0][facet_0][0][3] + sp[179] * PI1[facet_1][0][0][2] + sp[171] * PI4[0][facet_1][2][0] + sp[180] * PI2[facet_1][0][0][2] + sp[172] * PI5[0][facet_1][2][0];
    A[55] = sp[170] * PI0[facet_0][facet_1][3][1] + sp[167] * PI1[facet_0][0][3][1] + sp[161] * PI3[facet_1][facet_0][1][3] + sp[157] * PI4[0][facet_0][1][3] + sp[179] * PI1[facet_1][0][1][2] + sp[180] * PI2[facet_1][0][1][2];
    A[56] = sp[169] * PI3[facet_0][facet_1][3][1] + sp[165] * PI1[facet_0][0][3][1] + sp[160] * PI6[facet_0][facet_1][3][1] + sp[155] * PI4[0][facet_0][1][3] + sp[171] * PI4[0][facet_1][2][1] + sp[172] * PI5[0][facet_1][2][1];
    A[57] = sp[170] * PI0[facet_0][facet_1][3][2] + sp[169] * PI3[facet_0][facet_1][3][2] + sp[166] * PI2[facet_0][0][3][1] + sp[161] * PI3[facet_1][facet_0][2][3] + sp[160] * PI6[facet_0][facet_1][3][2] + sp[156] * PI5[0][facet_0][1][3] + sp[179] * PI1[facet_1][0][2][2] + sp[171] * PI4[0][facet_1][2][2] + sp[180] * PI2[facet_1][0][2][2] + sp[172] * PI5[0][facet_1][2][2];
    A[58] = sp[170] * PI0[facet_0][facet_1][3][3] + sp[169] * PI3[facet_0][facet_1][3][3] + sp[165] * PI1[facet_0][0][3][2] + sp[166] * PI2[facet_0][0][3][2] + sp[161] * PI3[facet_1][facet_0][3][3] + sp[160] * PI6[facet_0][facet_1][3][3] + sp[155] * PI4[0][facet_0][2][3] + sp[156] * PI5[0][facet_0][2][3] + sp[179] * PI1[facet_1][0][3][2] + sp[171] * PI4[0][facet_1][2][3] + sp[180] * PI2[facet_1][0][3][2] + sp[172] * PI5[0][facet_1][2][3];
    A[59] = sp[170] * PI0[facet_0][facet_1][3][4] + sp[167] * PI1[facet_0][0][3][2] + sp[169] * PI3[facet_0][facet_1][3][4] + sp[166] * PI2[facet_0][0][3][3] + sp[161] * PI3[facet_1][facet_0][4][3] + sp[157] * PI4[0][facet_0][2][3] + sp[160] * PI6[facet_0][facet_1][3][4] + sp[156] * PI5[0][facet_0][3][3] + sp[179] * PI1[facet_1][0][4][2] + sp[171] * PI4[0][facet_1][2][4] + sp[180] * PI2[facet_1][0][4][2] + sp[172] * PI5[0][facet_1][2][4];
    A[60] = sp[168] * PI0[facet_0][facet_0][4][0] + sp[164] * PI1[facet_0][0][4][0] + sp[159] * PI3[facet_0][facet_0][4][0] + sp[162] * PI1[facet_0][0][4][0] + sp[163] * PI2[facet_0][0][4][0] + sp[164] * PI1[facet_0][0][0][2] + sp[154] * PI4[0][facet_0][2][0] + sp[159] * PI3[facet_0][facet_0][0][4] + sp[154] * PI4[0][facet_0][0][4] + sp[158] * PI6[facet_0][facet_0][4][0] + sp[152] * PI4[0][facet_0][0][4] + sp[153] * PI5[0][facet_0][0][4] + sp[163] * PI2[facet_0][0][0][3] + sp[153] * PI5[0][facet_0][3][0];
    A[61] = sp[168] * PI0[facet_0][facet_0][4][1] + sp[164] * PI1[facet_0][0][4][1] + sp[164] * PI1[facet_0][0][1][2] + sp[159] * PI3[facet_0][facet_0][1][4] + sp[154] * PI4[0][facet_0][1][4] + sp[163] * PI2[facet_0][0][1][3];
    A[62] = sp[159] * PI3[facet_0][facet_0][4][1] + sp[162] * PI1[facet_0][0][4][1] + sp[154] * PI4[0][facet_0][2][1] + sp[158] * PI6[facet_0][facet_0][4][1] + sp[152] * PI4[0][facet_0][1][4] + sp[153] * PI5[0][facet_0][3][1];
    A[63] = sp[168] * PI0[facet_0][facet_0][4][2] + sp[159] * PI3[facet_0][facet_0][4][2] + sp[163] * PI2[facet_0][0][4][1] + sp[164] * PI1[facet_0][0][2][2] + sp[154] * PI4[0][facet_0][2][2] + sp[159] * PI3[facet_0][facet_0][2][4] + sp[158] * PI6[facet_0][facet_0][4][2] + sp[153] * PI5[0][facet_0][1][4] + sp[163] * PI2[facet_0][0][2][3] + sp[153] * PI5[0][facet_0][3][2];
    A[64] = sp[168] * PI0[facet_0][facet_0][4][3] + sp[159] * PI3[facet_0][facet_0][4][3] + sp[162] * PI1[facet_0][0][4][2] + sp[163] * PI2[facet_0][0][4][2] + sp[164] * PI1[facet_0][0][3][2] + sp[154] * PI4[0][facet_0][2][3] + sp[159] * PI3[facet_0][facet_0][3][4] + sp[158] * PI6[facet_0][facet_0][4][3] + sp[152] * PI4[0][facet_0][2][4] + sp[153] * PI5[0][facet_0][2][4] + sp[163] * PI2[facet_0][0][3][3] + sp[153] * PI5[0][facet_0][3][3];
    A[65] = sp[168] * PI0[facet_0][facet_0][4][4] + sp[164] * PI1[facet_0][0][4][2] + sp[159] * PI3[facet_0][facet_0][4][4] + sp[163] * PI2[facet_0][0][4][3] + sp[164] * PI1[facet_0][0][4][2] + sp[154] * PI4[0][facet_0][2][4] + sp[159] * PI3[facet_0][facet_0][4][4] + sp[154] * PI4[0][facet_0][2][4] + sp[158] * PI6[facet_0][facet_0][4][4] + sp[153] * PI5[0][facet_0][3][4] + sp[163] * PI2[facet_0][0][4][3] + sp[153] * PI5[0][facet_0][3][4];
    A[66] = sp[170] * PI0[facet_0][facet_1][4][0] + sp[167] * PI1[facet_0][0][4][0] + sp[169] * PI3[facet_0][facet_1][4][0] + sp[165] * PI1[facet_0][0][4][0] + sp[166] * PI2[facet_0][0][4][0] + sp[181] * PI1[facet_1][0][0][2] + sp[173] * PI4[0][facet_1][2][0] + sp[161] * PI3[facet_1][facet_0][0][4] + sp[157] * PI4[0][facet_0][0][4] + sp[160] * PI6[facet_0][facet_1][4][0] + sp[155] * PI4[0][facet_0][0][4] + sp[156] * PI5[0][facet_0][0][4] + sp[180] * PI2[facet_1][0][0][3] + sp[172] * PI5[0][facet_1][3][0];
    A[67] = sp[170] * PI0[facet_0][facet_1][4][1] + sp[167] * PI1[facet_0][0][4][1] + sp[181] * PI1[facet_1][0][1][2] + sp[161] * PI3[facet_1][facet_0][1][4] + sp[157] * PI4[0][facet_0][1][4] + sp[180] * PI2[facet_1][0][1][3];
    A[68] = sp[169] * PI3[facet_0][facet_1][4][1] + sp[165] * PI1[facet_0][0][4][1] + sp[173] * PI4[0][facet_1][2][1] + sp[160] * PI6[facet_0][facet_1][4][1] + sp[155] * PI4[0][facet_0][1][4] + sp[172] * PI5[0][facet_1][3][1];
    A[69] = sp[170] * PI0[facet_0][facet_1][4][2] + sp[169] * PI3[facet_0][facet_1][4][2] + sp[166] * PI2[facet_0][0][4][1] + sp[181] * PI1[facet_1][0][2][2] + sp[173] * PI4[0][facet_1][2][2] + sp[161] * PI3[facet_1][facet_0][2][4] + sp[160] * PI6[facet_0][facet_1][4][2] + sp[156] * PI5[0][facet_0][1][4] + sp[180] * PI2[facet_1][0][2][3] + sp[172] * PI5[0][facet_1][3][2];
    A[70] = sp[170] * PI0[facet_0][facet_1][4][3] + sp[169] * PI3[facet_0][facet_1][4][3] + sp[165] * PI1[facet_0][0][4][2] + sp[166] * PI2[facet_0][0][4][2] + sp[181] * PI1[facet_1][0][3][2] + sp[173] * PI4[0][facet_1][2][3] + sp[161] * PI3[facet_1][facet_0][3][4] + sp[160] * PI6[facet_0][facet_1][4][3] + sp[155] * PI4[0][facet_0][2][4] + sp[156] * PI5[0][facet_0][2][4] + sp[180] * PI2[facet_1][0][3][3] + sp[172] * PI5[0][facet_1][3][3];
    A[71] = sp[170] * PI0[facet_0][facet_1][4][4] + sp[167] * PI1[facet_0][0][4][2] + sp[169] * PI3[facet_0][facet_1][4][4] + sp[166] * PI2[facet_0][0][4][3] + sp[181] * PI1[facet_1][0][4][2] + sp[173] * PI4[0][facet_1][2][4] + sp[161] * PI3[facet_1][facet_0][4][4] + sp[157] * PI4[0][facet_0][2][4] + sp[160] * PI6[facet_0][facet_1][4][4] + sp[156] * PI5[0][facet_0][3][4] + sp[180] * PI2[facet_1][0][4][3] + sp[172] * PI5[0][facet_1][3][4];
    A[72] = sp[170] * PI0[facet_1][facet_0][0][0] + sp[181] * PI1[facet_1][0][0][0] + sp[161] * PI3[facet_1][facet_0][0][0] + sp[179] * PI1[facet_1][0][0][0] + sp[180] * PI2[facet_1][0][0][0] + sp[167] * PI1[facet_0][0][0][0] + sp[157] * PI4[0][facet_0][0][0] + sp[169] * PI3[facet_0][facet_1][0][0] + sp[173] * PI4[0][facet_1][0][0] + sp[160] * PI6[facet_1][facet_0][0][0] + sp[171] * PI4[0][facet_1][0][0] + sp[172] * PI5[0][facet_1][0][0] + sp[165] * PI1[facet_0][0][0][0] + sp[155] * PI4[0][facet_0][0][0] + sp[166] * PI2[facet_0][0][0][0] + sp[156] * PI5[0][facet_0][0][0];
    A[73] = sp[170] * PI0[facet_1][facet_0][0][1] + sp[181] * PI1[facet_1][0][0][1] + sp[167] * PI1[facet_0][0][1][0] + sp[169] * PI3[facet_0][facet_1][1][0] + sp[173] * PI4[0][facet_1][1][0] + sp[165] * PI1[facet_0][0][1][0] + sp[166] * PI2[facet_0][0][1][0];
    A[74] = sp[161] * PI3[facet_1][facet_0][0][1] + sp[179] * PI1[facet_1][0][0][1] + sp[157] * PI4[0][facet_0][0][1] + sp[160] * PI6[facet_1][facet_0][0][1] + sp[171] * PI4[0][facet_1][1][0] + sp[155] * PI4[0][facet_0][0][1] + sp[156] * PI5[0][facet_0][0][1];
    A[75] = sp[170] * PI0[facet_1][facet_0][0][2] + sp[161] * PI3[facet_1][facet_0][0][2] + sp[180] * PI2[facet_1][0][0][1] + sp[167] * PI1[facet_0][0][2][0] + sp[157] * PI4[0][facet_0][0][2] + sp[169] * PI3[facet_0][facet_1][2][0] + sp[160] * PI6[facet_1][facet_0][0][2] + sp[172] * PI5[0][facet_1][1][0] + sp[165] * PI1[facet_0][0][2][0] + sp[155] * PI4[0][facet_0][0][2] + sp[166] * PI2[facet_0][0][2][0] + sp[156] * PI5[0][facet_0][0][2];
    A[76] = sp[170] * PI0[facet_1][facet_0][0][3] + sp[161] * PI3[facet_1][facet_0][0][3] + sp[179] * PI1[facet_1][0][0][2] + sp[180] * PI2[facet_1][0][0][2] + sp[167] * PI1[facet_0][0][3][0] + sp[157] * PI4[0][facet_0][0][3] + sp[169] * PI3[facet_0][facet_1][3][0] + sp[160] * PI6[facet_1][facet_0][0][3] + sp[171] * PI4[0][facet_1][2][0] + sp[172] * PI5[0][facet_1][2][0] + sp[165] * PI1[facet_0][0][3][0] + sp[155] * PI4[0][facet_0][0][3] + sp[166] * PI2[facet_0][0][3][0] + sp[156] * PI5[0][facet_0][0][3];
    A[77] = sp[170] * PI0[facet_1][facet_0][0][4] + sp[181] * PI1[facet_1][0][0][2] + sp[161] * PI3[facet_1][facet_0][0][4] + sp[180] * PI2[facet_1][0][0][3] + sp[167] * PI1[facet_0][0][4][0] + sp[157] * PI4[0][facet_0][0][4] + sp[169] * PI3[facet_0][facet_1][4][0] + sp[173] * PI4[0][facet_1][2][0] + sp[160] * PI6[facet_1][facet_0][0][4] + sp[172] * PI5[0][facet_1][3][0] + sp[165] * PI1[facet_0][0][4][0] + sp[155] * PI4[0][facet_0][0][4] + sp[166] * PI2[facet_0][0][4][0] + sp[156] * PI5[0][facet_0][0][4];
    A[78] = sp[185] * PI0[facet_1][facet_1][0][0] + sp[184] * PI1[facet_1][0][0][0] + sp[178] * PI3[facet_1][facet_1][0][0] + sp[182] * PI1[facet_1][0][0][0] + sp[183] * PI2[facet_1][0][0][0] + sp[184] * PI1[facet_1][0][0][0] + sp[176] * PI4[0][facet_1][0][0] + sp[178] * PI3[facet_1][facet_1][0][0] + sp[176] * PI4[0][facet_1][0][0] + sp[177] * PI6[facet_1][facet_1][0][0] + sp[174] * PI4[0][facet_1][0][0] + sp[175] * PI5[0][facet_1][0][0] + sp[182] * PI1[facet_1][0][0][0] + sp[174] * PI4[0][facet_1][0][0] + sp[183] * PI2[facet_1][0][0][0] + sp[175] * PI5[0][facet_1][0][0];
    A[79] = sp[185] * PI0[facet_1][facet_1][0][1] + sp[184] * PI1[facet_1][0][0][1] + sp[184] * PI1[facet_1][0][1][0] + sp[178] * PI3[facet_1][facet_1][1][0] + sp[176] * PI4[0][facet_1][1][0] + sp[182] * PI1[facet_1][0][1][0] + sp[183] * PI2[facet_1][0][1][0];
    A[80] = sp[178] * PI3[facet_1][facet_1][0][1] + sp[182] * PI1[facet_1][0][0][1] + sp[176] * PI4[0][facet_1][0][1] + sp[177] * PI6[facet_1][facet_1][0][1] + sp[174] * PI4[0][facet_1][1][0] + sp[174] * PI4[0][facet_1][0][1] + sp[175] * PI5[0][facet_1][0][1];
    A[81] = sp[185] * PI0[facet_1][facet_1][0][2] + sp[178] * PI3[facet_1][facet_1][0][2] + sp[183] * PI2[facet_1][0][0][1] + sp[184] * PI1[facet_1][0][2][0] + sp[176] * PI4[0][facet_1][0][2] + sp[178] * PI3[facet_1][facet_1][2][0] + sp[177] * PI6[facet_1][facet_1][0][2] + sp[175] * PI5[0][facet_1][1][0] + sp[182] * PI1[facet_1][0][2][0] + sp[174] * PI4[0][facet_1][0][2] + sp[183] * PI2[facet_1][0][2][0] + sp[175] * PI5[0][facet_1][0][2];
    A[82] = sp[185] * PI0[facet_1][facet_1][0][3] + sp[178] * PI3[facet_1][facet_1][0][3] + sp[182] * PI1[facet_1][0][0][2] + sp[183] * PI2[facet_1][0][0][2] + sp[184] * PI1[facet_1][0][3][0] + sp[176] * PI4[0][facet_1][0][3] + sp[178] * PI3[facet_1][facet_1][3][0] + sp[177] * PI6[facet_1][facet_1][0][3] + sp[174] * PI4[0][facet_1][2][0] + sp[175] * PI5[0][facet_1][2][0] + sp[182] * PI1[facet_1][0][3][0] + sp[174] * PI4[0][facet_1][0][3] + sp[183] * PI2[facet_1][0][3][0] + sp[175] * PI5[0][facet_1][0][3];
    A[83] = sp[185] * PI0[facet_1][facet_1][0][4] + sp[184] * PI1[facet_1][0][0][2] + sp[178] * PI3[facet_1][facet_1][0][4] + sp[183] * PI2[facet_1][0][0][3] + sp[184] * PI1[facet_1][0][4][0] + sp[176] * PI4[0][facet_1][0][4] + sp[178] * PI3[facet_1][facet_1][4][0] + sp[176] * PI4[0][facet_1][2][0] + sp[177] * PI6[facet_1][facet_1][0][4] + sp[175] * PI5[0][facet_1][3][0] + sp[182] * PI1[facet_1][0][4][0] + sp[174] * PI4[0][facet_1][0][4] + sp[183] * PI2[facet_1][0][4][0] + sp[175] * PI5[0][facet_1][0][4];
    A[84] = sp[170] * PI0[facet_1][facet_0][1][0] + sp[181] * PI1[facet_1][0][1][0] + sp[161] * PI3[facet_1][facet_0][1][0] + sp[179] * PI1[facet_1][0][1][0] + sp[180] * PI2[facet_1][0][1][0] + sp[167] * PI1[facet_0][0][0][1] + sp[157] * PI4[0][facet_0][1][0];
    A[85] = sp[170] * PI0[facet_1][facet_0][1][1] + sp[181] * PI1[facet_1][0][1][1] + sp[167] * PI1[facet_0][0][1][1];
    A[86] = sp[161] * PI3[facet_1][facet_0][1][1] + sp[179] * PI1[facet_1][0][1][1] + sp[157] * PI4[0][facet_0][1][1];
    A[87] = sp[170] * PI0[facet_1][facet_0][1][2] + sp[161] * PI3[facet_1][facet_0][1][2] + sp[180] * PI2[facet_1][0][1][1] + sp[167] * PI1[facet_0][0][2][1] + sp[157] * PI4[0][facet_0][1][2];
    A[88] = sp[170] * PI0[facet_1][facet_0][1][3] + sp[161] * PI3[facet_1][facet_0][1][3] + sp[179] * PI1[facet_1][0][1][2] + sp[180] * PI2[facet_1][0][1][2] + sp[167] * PI1[facet_0][0][3][1] + sp[157] * PI4[0][facet_0][1][3];
    A[89] = sp[170] * PI0[facet_1][facet_0][1][4] + sp[181] * PI1[facet_1][0][1][2] + sp[161] * PI3[facet_1][facet_0][1][4] + sp[180] * PI2[facet_1][0][1][3] + sp[167] * PI1[facet_0][0][4][1] + sp[157] * PI4[0][facet_0][1][4];
    A[90] = sp[185] * PI0[facet_1][facet_1][1][0] + sp[184] * PI1[facet_1][0][1][0] + sp[178] * PI3[facet_1][facet_1][1][0] + sp[182] * PI1[facet_1][0][1][0] + sp[183] * PI2[facet_1][0][1][0] + sp[184] * PI1[facet_1][0][0][1] + sp[176] * PI4[0][facet_1][1][0];
    A[91] = sp[185] * PI0[facet_1][facet_1][1][1] + sp[184] * PI1[facet_1][0][1][1] + sp[184] * PI1[facet_1][0][1][1];
    A[92] = sp[178] * PI3[facet_1][facet_1][1][1] + sp[182] * PI1[facet_1][0][1][1] + sp[176] * PI4[0][facet_1][1][1];
    A[93] = sp[185] * PI0[facet_1][facet_1][1][2] + sp[178] * PI3[facet_1][facet_1][1][2] + sp[183] * PI2[facet_1][0][1][1] + sp[184] * PI1[facet_1][0][2][1] + sp[176] * PI4[0][facet_1][1][2];
    A[94] = sp[185] * PI0[facet_1][facet_1][1][3] + sp[178] * PI3[facet_1][facet_1][1][3] + sp[182] * PI1[facet_1][0][1][2] + sp[183] * PI2[facet_1][0][1][2] + sp[184] * PI1[facet_1][0][3][1] + sp[176] * PI4[0][facet_1][1][3];
    A[95] = sp[185] * PI0[facet_1][facet_1][1][4] + sp[184] * PI1[facet_1][0][1][2] + sp[178] * PI3[facet_1][facet_1][1][4] + sp[183] * PI2[facet_1][0][1][3] + sp[184] * PI1[facet_1][0][4][1] + sp[176] * PI4[0][facet_1][1][4];
    A[96] = sp[169] * PI3[facet_0][facet_1][0][1] + sp[173] * PI4[0][facet_1][0][1] + sp[160] * PI6[facet_1][facet_0][1][0] + sp[171] * PI4[0][facet_1][0][1] + sp[172] * PI5[0][facet_1][0][1] + sp[165] * PI1[facet_0][0][0][1] + sp[155] * PI4[0][facet_0][1][0];
    A[97] = sp[169] * PI3[facet_0][facet_1][1][1] + sp[173] * PI4[0][facet_1][1][1] + sp[165] * PI1[facet_0][0][1][1];
    A[98] = sp[160] * PI6[facet_1][facet_0][1][1] + sp[171] * PI4[0][facet_1][1][1] + sp[155] * PI4[0][facet_0][1][1];
    A[99] = sp[169] * PI3[facet_0][facet_1][2][1] + sp[160] * PI6[facet_1][facet_0][1][2] + sp[172] * PI5[0][facet_1][1][1] + sp[165] * PI1[facet_0][0][2][1] + sp[155] * PI4[0][facet_0][1][2];
    A[100] = sp[169] * PI3[facet_0][facet_1][3][1] + sp[160] * PI6[facet_1][facet_0][1][3] + sp[171] * PI4[0][facet_1][2][1] + sp[172] * PI5[0][facet_1][2][1] + sp[165] * PI1[facet_0][0][3][1] + sp[155] * PI4[0][facet_0][1][3];
    A[101] = sp[169] * PI3[facet_0][facet_1][4][1] + sp[173] * PI4[0][facet_1][2][1] + sp[160] * PI6[facet_1][facet_0][1][4] + sp[172] * PI5[0][facet_1][3][1] + sp[165] * PI1[facet_0][0][4][1] + sp[155] * PI4[0][facet_0][1][4];
    A[102] = sp[178] * PI3[facet_1][facet_1][0][1] + sp[176] * PI4[0][facet_1][0][1] + sp[177] * PI6[facet_1][facet_1][1][0] + sp[174] * PI4[0][facet_1][0][1] + sp[175] * PI5[0][facet_1][0][1] + sp[182] * PI1[facet_1][0][0][1] + sp[174] * PI4[0][facet_1][1][0];
    A[103] = sp[178] * PI3[facet_1][facet_1][1][1] + sp[176] * PI4[0][facet_1][1][1] + sp[182] * PI1[facet_1][0][1][1];
    A[104] = sp[177] * PI6[facet_1][facet_1][1][1] + sp[174] * PI4[0][facet_1][1][1] + sp[174] * PI4[0][facet_1][1][1];
    A[105] = sp[178] * PI3[facet_1][facet_1][2][1] + sp[177] * PI6[facet_1][facet_1][1][2] + sp[175] * PI5[0][facet_1][1][1] + sp[182] * PI1[facet_1][0][2][1] + sp[174] * PI4[0][facet_1][1][2];
    A[106] = sp[178] * PI3[facet_1][facet_1][3][1] + sp[177] * PI6[facet_1][facet_1][1][3] + sp[174] * PI4[0][facet_1][2][1] + sp[175] * PI5[0][facet_1][2][1] + sp[182] * PI1[facet_1][0][3][1] + sp[174] * PI4[0][facet_1][1][3];
    A[107] = sp[178] * PI3[facet_1][facet_1][4][1] + sp[176] * PI4[0][facet_1][2][1] + sp[177] * PI6[facet_1][facet_1][1][4] + sp[175] * PI5[0][facet_1][3][1] + sp[182] * PI1[facet_1][0][4][1] + sp[174] * PI4[0][facet_1][1][4];
    A[108] = sp[170] * PI0[facet_1][facet_0][2][0] + sp[181] * PI1[facet_1][0][2][0] + sp[161] * PI3[facet_1][facet_0][2][0] + sp[179] * PI1[facet_1][0][2][0] + sp[180] * PI2[facet_1][0][2][0] + sp[169] * PI3[facet_0][facet_1][0][2] + sp[173] * PI4[0][facet_1][0][2] + sp[160] * PI6[facet_1][facet_0][2][0] + sp[171] * PI4[0][facet_1][0][2] + sp[172] * PI5[0][facet_1][0][2] + sp[166] * PI2[facet_0][0][0][1] + sp[156] * PI5[0][facet_0][1][0];
    A[109] = sp[170] * PI0[facet_1][facet_0][2][1] + sp[181] * PI1[facet_1][0][2][1] + sp[169] * PI3[facet_0][facet_1][1][2] + sp[173] * PI4[0][facet_1][1][2] + sp[166] * PI2[facet_0][0][1][1];
    A[110] = sp[161] * PI3[facet_1][facet_0][2][1] + sp[179] * PI1[facet_1][0][2][1] + sp[160] * PI6[facet_1][facet_0][2][1] + sp[171] * PI4[0][facet_1][1][2] + sp[156] * PI5[0][facet_0][1][1];
    A[111] = sp[170] * PI0[facet_1][facet_0][2][2] + sp[161] * PI3[facet_1][facet_0][2][2] + sp[180] * PI2[facet_1][0][2][1] + sp[169] * PI3[facet_0][facet_1][2][2] + sp[160] * PI6[facet_1][facet_0][2][2] + sp[172] * PI5[0][facet_1][1][2] + sp[166] * PI2[facet_0][0][2][1] + sp[156] * PI5[0][facet_0][1][2];
    A[112] = sp[170] * PI0[facet_1][facet_0][2][3] + sp[161] * PI3[facet_1][facet_0][2][3] + sp[179] * PI1[facet_1][0][2][2] + sp[180] * PI2[facet_1][0][2][2] + sp[169] * PI3[facet_0][facet_1][3][2] + sp[160] * PI6[facet_1][facet_0][2][3] + sp[171] * PI4[0][facet_1][2][2] + sp[172] * PI5[0][facet_1][2][2] + sp[166] * PI2[facet_0][0][3][1] + sp[156] * PI5[0][facet_0][1][3];
    A[113] = sp[170] * PI0[facet_1][facet_0][2][4] + sp[181] * PI1[facet_1][0][2][2] + sp[161] * PI3[facet_1][facet_0][2][4] + sp[180] * PI2[facet_1][0][2][3] + sp[169] * PI3[facet_0][facet_1][4][2] + sp[173] * PI4[0][facet_1][2][2] + sp[160] * PI6[facet_1][facet_0][2][4] + sp[172] * PI5[0][facet_1][3][2] + sp[166] * PI2[facet_0][0][4][1] + sp[156] * PI5[0][facet_0][1][4];
    A[114] = sp[185] * PI0[facet_1][facet_1][2][0] + sp[184] * PI1[facet_1][0][2][0] + sp[178] * PI3[facet_1][facet_1][2][0] + sp[182] * PI1[facet_1][0][2][0] + sp[183] * PI2[facet_1][0][2][0] + sp[178] * PI3[facet_1][facet_1][0][2] + sp[176] * PI4[0][facet_1][0][2] + sp[177] * PI6[facet_1][facet_1][2][0] + sp[174] * PI4[0][facet_1][0][2] + sp[175] * PI5[0][facet_1][0][2] + sp[183] * PI2[facet_1][0][0][1] + sp[175] * PI5[0][facet_1][1][0];
    A[115] = sp[185] * PI0[facet_1][facet_1][2][1] + sp[184] * PI1[facet_1][0][2][1] + sp[178] * PI3[facet_1][facet_1][1][2] + sp[176] * PI4[0][facet_1][1][2] + sp[183] * PI2[facet_1][0][1][1];
    A[116] = sp[178] * PI3[facet_1][facet_1][2][1] + sp[182] * PI1[facet_1][0][2][1] + sp[177] * PI6[facet_1][facet_1][2][1] + sp[174] * PI4[0][facet_1][1][2] + sp[175] * PI5[0][facet_1][1][1];
    A[117] = sp[185] * PI0[facet_1][facet_1][2][2] + sp[178] * PI3[facet_1][facet_1][2][2] + sp[183] * PI2[facet_1][0][2][1] + sp[178] * PI3[facet_1][facet_1][2][2] + sp[177] * PI6[facet_1][facet_1][2][2] + sp[175] * PI5[0][facet_1][1][2] + sp[183] * PI2[facet_1][0][2][1] + sp[175] * PI5[0][facet_1][1][2];
    A[118] = sp[185] * PI0[facet_1][facet_1][2][3] + sp[178] * PI3[facet_1][facet_1][2][3] + sp[182] * PI1[facet_1][0][2][2] + sp[183] * PI2[facet_1][0][2][2] + sp[178] * PI3[facet_1][facet_1][3][2] + sp[177] * PI6[facet_1][facet_1][2][3] + sp[174] * PI4[0][facet_1][2][2] + sp[175] * PI5[0][facet_1][2][2] + sp[183] * PI2[facet_1][0][3][1] + sp[175] * PI5[0][facet_1][1][3];
    A[119] = sp[185] * PI0[facet_1][facet_1][2][4] + sp[184] * PI1[facet_1][0][2][2] + sp[178] * PI3[facet_1][facet_1][2][4] + sp[183] * PI2[facet_1][0][2][3] + sp[178] * PI3[facet_1][facet_1][4][2] + sp[176] * PI4[0][facet_1][2][2] + sp[177] * PI6[facet_1][facet_1][2][4] + sp[175] * PI5[0][facet_1][3][2] + sp[183] * PI2[facet_1][0][4][1] + sp[175] * PI5[0][facet_1][1][4];
    A[120] = sp[170] * PI0[facet_1][facet_0][3][0] + sp[181] * PI1[facet_1][0][3][0] + sp[161] * PI3[facet_1][facet_0][3][0] + sp[179] * PI1[facet_1][0][3][0] + sp[180] * PI2[facet_1][0][3][0] + sp[169] * PI3[facet_0][facet_1][0][3] + sp[173] * PI4[0][facet_1][0][3] + sp[160] * PI6[facet_1][facet_0][3][0] + sp[171] * PI4[0][facet_1][0][3] + sp[172] * PI5[0][facet_1][0][3] + sp[165] * PI1[facet_0][0][0][2] + sp[155] * PI4[0][facet_0][2][0] + sp[166] * PI2[facet_0][0][0][2] + sp[156] * PI5[0][facet_0][2][0];
    A[121] = sp[170] * PI0[facet_1][facet_0][3][1] + sp[181] * PI1[facet_1][0][3][1] + sp[169] * PI3[facet_0][facet_1][1][3] + sp[173] * PI4[0][facet_1][1][3] + sp[165] * PI1[facet_0][0][1][2] + sp[166] * PI2[facet_0][0][1][2];
    A[122] = sp[161] * PI3[facet_1][facet_0][3][1] + sp[179] * PI1[facet_1][0][3][1] + sp[160] * PI6[facet_1][facet_0][3][1] + sp[171] * PI4[0][facet_1][1][3] + sp[155] * PI4[0][facet_0][2][1] + sp[156] * PI5[0][facet_0][2][1];
    A[123] = sp[170] * PI0[facet_1][facet_0][3][2] + sp[161] * PI3[facet_1][facet_0][3][2] + sp[180] * PI2[facet_1][0][3][1] + sp[169] * PI3[facet_0][facet_1][2][3] + sp[160] * PI6[facet_1][facet_0][3][2] + sp[172] * PI5[0][facet_1][1][3] + sp[165] * PI1[facet_0][0][2][2] + sp[155] * PI4[0][facet_0][2][2] + sp[166] * PI2[facet_0][0][2][2] + sp[156] * PI5[0][facet_0][2][2];
    A[124] = sp[170] * PI0[facet_1][facet_0][3][3] + sp[161] * PI3[facet_1][facet_0][3][3] + sp[179] * PI1[facet_1][0][3][2] + sp[180] * PI2[facet_1][0][3][2] + sp[169] * PI3[facet_0][facet_1][3][3] + sp[160] * PI6[facet_1][facet_0][3][3] + sp[171] * PI4[0][facet_1][2][3] + sp[172] * PI5[0][facet_1][2][3] + sp[165] * PI1[facet_0][0][3][2] + sp[155] * PI4[0][facet_0][2][3] + sp[166] * PI2[facet_0][0][3][2] + sp[156] * PI5[0][facet_0][2][3];
    A[125] = sp[170] * PI0[facet_1][facet_0][3][4] + sp[181] * PI1[facet_1][0][3][2] + sp[161] * PI3[facet_1][facet_0][3][4] + sp[180] * PI2[facet_1][0][3][3] + sp[169] * PI3[facet_0][facet_1][4][3] + sp[173] * PI4[0][facet_1][2][3] + sp[160] * PI6[facet_1][facet_0][3][4] + sp[172] * PI5[0][facet_1][3][3] + sp[165] * PI1[facet_0][0][4][2] + sp[155] * PI4[0][facet_0][2][4] + sp[166] * PI2[facet_0][0][4][2] + sp[156] * PI5[0][facet_0][2][4];
    A[126] = sp[185] * PI0[facet_1][facet_1][3][0] + sp[184] * PI1[facet_1][0][3][0] + sp[178] * PI3[facet_1][facet_1][3][0] + sp[182] * PI1[facet_1][0][3][0] + sp[183] * PI2[facet_1][0][3][0] + sp[178] * PI3[facet_1][facet_1][0][3] + sp[176] * PI4[0][facet_1][0][3] + sp[177] * PI6[facet_1][facet_1][3][0] + sp[174] * PI4[0][facet_1][0][3] + sp[175] * PI5[0][facet_1][0][3] + sp[182] * PI1[facet_1][0][0][2] + sp[174] * PI4[0][facet_1][2][0] + sp[183] * PI2[facet_1][0][0][2] + sp[175] * PI5[0][facet_1][2][0];
    A[127] = sp[185] * PI0[facet_1][facet_1][3][1] + sp[184] * PI1[facet_1][0][3][1] + sp[178] * PI3[facet_1][facet_1][1][3] + sp[176] * PI4[0][facet_1][1][3] + sp[182] * PI1[facet_1][0][1][2] + sp[183] * PI2[facet_1][0][1][2];
    A[128] = sp[178] * PI3[facet_1][facet_1][3][1] + sp[182] * PI1[facet_1][0][3][1] + sp[177] * PI6[facet_1][facet_1][3][1] + sp[174] * PI4[0][facet_1][1][3] + sp[174] * PI4[0][facet_1][2][1] + sp[175] * PI5[0][facet_1][2][1];
    A[129] = sp[185] * PI0[facet_1][facet_1][3][2] + sp[178] * PI3[facet_1][facet_1][3][2] + sp[183] * PI2[facet_1][0][3][1] + sp[178] * PI3[facet_1][facet_1][2][3] + sp[177] * PI6[facet_1][facet_1][3][2] + sp[175] * PI5[0][facet_1][1][3] + sp[182] * PI1[facet_1][0][2][2] + sp[174] * PI4[0][facet_1][2][2] + sp[183] * PI2[facet_1][0][2][2] + sp[175] * PI5[0][facet_1][2][2];
    A[130] = sp[185] * PI0[facet_1][facet_1][3][3] + sp[178] * PI3[facet_1][facet_1][3][3] + sp[182] * PI1[facet_1][0][3][2] + sp[183] * PI2[facet_1][0][3][2] + sp[178] * PI3[facet_1][facet_1][3][3] + sp[177] * PI6[facet_1][facet_1][3][3] + sp[174] * PI4[0][facet_1][2][3] + sp[175] * PI5[0][facet_1][2][3] + sp[182] * PI1[facet_1][0][3][2] + sp[174] * PI4[0][facet_1][2][3] + sp[183] * PI2[facet_1][0][3][2] + sp[175] * PI5[0][facet_1][2][3];
    A[131] = sp[185] * PI0[facet_1][facet_1][3][4] + sp[184] * PI1[facet_1][0][3][2] + sp[178] * PI3[facet_1][facet_1][3][4] + sp[183] * PI2[facet_1][0][3][3] + sp[178] * PI3[facet_1][facet_1][4][3] + sp[176] * PI4[0][facet_1][2][3] + sp[177] * PI6[facet_1][facet_1][3][4] + sp[175] * PI5[0][facet_1][3][3] + sp[182] * PI1[facet_1][0][4][2] + sp[174] * PI4[0][facet_1][2][4] + sp[183] * PI2[facet_1][0][4][2] + sp[175] * PI5[0][facet_1][2][4];
    A[132] = sp[170] * PI0[facet_1][facet_0][4][0] + sp[181] * PI1[facet_1][0][4][0] + sp[161] * PI3[facet_1][facet_0][4][0] + sp[179] * PI1[facet_1][0][4][0] + sp[180] * PI2[facet_1][0][4][0] + sp[167] * PI1[facet_0][0][0][2] + sp[157] * PI4[0][facet_0][2][0] + sp[169] * PI3[facet_0][facet_1][0][4] + sp[173] * PI4[0][facet_1][0][4] + sp[160] * PI6[facet_1][facet_0][4][0] + sp[171] * PI4[0][facet_1][0][4] + sp[172] * PI5[0][facet_1][0][4] + sp[166] * PI2[facet_0][0][0][3] + sp[156] * PI5[0][facet_0][3][0];
    A[133] = sp[170] * PI0[facet_1][facet_0][4][1] + sp[181] * PI1[facet_1][0][4][1] + sp[167] * PI1[facet_0][0][1][2] + sp[169] * PI3[facet_0][facet_1][1][4] + sp[173] * PI4[0][facet_1][1][4] + sp[166] * PI2[facet_0][0][1][3];
    A[134] = sp[161] * PI3[facet_1][facet_0][4][1] + sp[179] * PI1[facet_1][0][4][1] + sp[157] * PI4[0][facet_0][2][1] + sp[160] * PI6[facet_1][facet_0][4][1] + sp[171] * PI4[0][facet_1][1][4] + sp[156] * PI5[0][facet_0][3][1];
    A[135] = sp[170] * PI0[facet_1][facet_0][4][2] + sp[161] * PI3[facet_1][facet_0][4][2] + sp[180] * PI2[facet_1][0][4][1] + sp[167] * PI1[facet_0][0][2][2] + sp[157] * PI4[0][facet_0][2][2] + sp[169] * PI3[facet_0][facet_1][2][4] + sp[160] * PI6[facet_1][facet_0][4][2] + sp[172] * PI5[0][facet_1][1][4] + sp[166] * PI2[facet_0][0][2][3] + sp[156] * PI5[0][facet_0][3][2];
    A[136] = sp[170] * PI0[facet_1][facet_0][4][3] + sp[161] * PI3[facet_1][facet_0][4][3] + sp[179] * PI1[facet_1][0][4][2] + sp[180] * PI2[facet_1][0][4][2] + sp[167] * PI1[facet_0][0][3][2] + sp[157] * PI4[0][facet_0][2][3] + sp[169] * PI3[facet_0][facet_1][3][4] + sp[160] * PI6[facet_1][facet_0][4][3] + sp[171] * PI4[0][facet_1][2][4] + sp[172] * PI5[0][facet_1][2][4] + sp[166] * PI2[facet_0][0][3][3] + sp[156] * PI5[0][facet_0][3][3];
    A[137] = sp[170] * PI0[facet_1][facet_0][4][4] + sp[181] * PI1[facet_1][0][4][2] + sp[161] * PI3[facet_1][facet_0][4][4] + sp[180] * PI2[facet_1][0][4][3] + sp[167] * PI1[facet_0][0][4][2] + sp[157] * PI4[0][facet_0][2][4] + sp[169] * PI3[facet_0][facet_1][4][4] + sp[173] * PI4[0][facet_1][2][4] + sp[160] * PI6[facet_1][facet_0][4][4] + sp[172] * PI5[0][facet_1][3][4] + sp[166] * PI2[facet_0][0][4][3] + sp[156] * PI5[0][facet_0][3][4];
    A[138] = sp[185] * PI0[facet_1][facet_1][4][0] + sp[184] * PI1[facet_1][0][4][0] + sp[178] * PI3[facet_1][facet_1][4][0] + sp[182] * PI1[facet_1][0][4][0] + sp[183] * PI2[facet_1][0][4][0] + sp[184] * PI1[facet_1][0][0][2] + sp[176] * PI4[0][facet_1][2][0] + sp[178] * PI3[facet_1][facet_1][0][4] + sp[176] * PI4[0][facet_1][0][4] + sp[177] * PI6[facet_1][facet_1][4][0] + sp[174] * PI4[0][facet_1][0][4] + sp[175] * PI5[0][facet_1][0][4] + sp[183] * PI2[facet_1][0][0][3] + sp[175] * PI5[0][facet_1][3][0];
    A[139] = sp[185] * PI0[facet_1][facet_1][4][1] + sp[184] * PI1[facet_1][0][4][1] + sp[184] * PI1[facet_1][0][1][2] + sp[178] * PI3[facet_1][facet_1][1][4] + sp[176] * PI4[0][facet_1][1][4] + sp[183] * PI2[facet_1][0][1][3];
    A[140] = sp[178] * PI3[facet_1][facet_1][4][1] + sp[182] * PI1[facet_1][0][4][1] + sp[176] * PI4[0][facet_1][2][1] + sp[177] * PI6[facet_1][facet_1][4][1] + sp[174] * PI4[0][facet_1][1][4] + sp[175] * PI5[0][facet_1][3][1];
    A[141] = sp[185] * PI0[facet_1][facet_1][4][2] + sp[178] * PI3[facet_1][facet_1][4][2] + sp[183] * PI2[facet_1][0][4][1] + sp[184] * PI1[facet_1][0][2][2] + sp[176] * PI4[0][facet_1][2][2] + sp[178] * PI3[facet_1][facet_1][2][4] + sp[177] * PI6[facet_1][facet_1][4][2] + sp[175] * PI5[0][facet_1][1][4] + sp[183] * PI2[facet_1][0][2][3] + sp[175] * PI5[0][facet_1][3][2];
    A[142] = sp[185] * PI0[facet_1][facet_1][4][3] + sp[178] * PI3[facet_1][facet_1][4][3] + sp[182] * PI1[facet_1][0][4][2] + sp[183] * PI2[facet_1][0][4][2] + sp[184] * PI1[facet_1][0][3][2] + sp[176] * PI4[0][facet_1][2][3] + sp[178] * PI3[facet_1][facet_1][3][4] + sp[177] * PI6[facet_1][facet_1][4][3] + sp[174] * PI4[0][facet_1][2][4] + sp[175] * PI5[0][facet_1][2][4] + sp[183] * PI2[facet_1][0][3][3] + sp[175] * PI5[0][facet_1][3][3];
    A[143] = sp[185] * PI0[facet_1][facet_1][4][4] + sp[184] * PI1[facet_1][0][4][2] + sp[178] * PI3[facet_1][facet_1][4][4] + sp[183] * PI2[facet_1][0][4][3] + sp[184] * PI1[facet_1][0][4][2] + sp[176] * PI4[0][facet_1][2][4] + sp[178] * PI3[facet_1][facet_1][4][4] + sp[176] * PI4[0][facet_1][2][4] + sp[177] * PI6[facet_1][facet_1][4][4] + sp[175] * PI5[0][facet_1][3][4] + sp[183] * PI2[facet_1][0][4][3] + sp[175] * PI5[0][facet_1][3][4];
  }

};


class biharmonic_cell_integral_1_otherwise: public ufc::cell_integral
{
public:

  biharmonic_cell_integral_1_otherwise() : ufc::cell_integral()
  {

  }

  ~biharmonic_cell_integral_1_otherwise() override
  {

  }

  const std::vector<bool> & enabled_coefficients() const final override
  {
static const std::vector<bool> enabled({true});
return enabled;
  }

  void tabulate_tensor(double * A,
                       const double * const * w,
                       const double * coordinate_dofs,
                       int cell_orientation,
                       std::size_t local_facet) const final override
  {
    // This function was generated using 'uflacs' representation
    // with the following integrals metadata:
    // 
    // num_cells:         None
    // optimize:          True
    // precision:         16
    // quadrature_degree: 4
    // quadrature_rule:   'default'
    // representation:    'uflacs'
    // 
    // and the following integral 0 metadata:
    // 
    // estimated_polynomial_degree: 4
    // optimize:                    True
    // precision:                   16
    // quadrature_degree:           4
    // quadrature_rule:             'default'
    // representation:              'uflacs'
    // Quadrature rules
    alignas(32) static const double weights6[6] = { 0.054975871827661, 0.054975871827661, 0.054975871827661, 0.1116907948390055, 0.1116907948390055, 0.1116907948390055 };
    // Precomputed values of basis functions and precomputations
    // FE* dimensions: [entities][points][dofs]
    // PI* dimensions: [entities][dofs][dofs] or [entities][dofs]
    // PM* dimensions: [entities][dofs][dofs]
    alignas(32) static const double FE3_C0_D01_Q6[1][1][2] = { { { -1.0, 1.0 } } };
    alignas(32) static const double FE6_C0_Q6[1][6][6] =
        { { { -0.07480380774819603, 0.5176323419876735, -0.07480380774819664, 0.2992152309927869, 0.03354481152314828, 0.2992152309927838 },
            { -0.07480380774819599, -0.07480380774819662, 0.5176323419876736, 0.2992152309927869, 0.2992152309927838, 0.03354481152314827 },
            { 0.5176323419876714, -0.07480380774819663, -0.07480380774819664, 0.03354481152314866, 0.2992152309927866, 0.2992152309927866 },
            { -0.04820837781551188, -0.08473049309397782, -0.04820837781551189, 0.1928335112620478, 0.795480226200906, 0.1928335112620478 },
            { -0.04820837781551192, -0.04820837781551197, -0.08473049309397784, 0.1928335112620478, 0.1928335112620478, 0.7954802262009062 },
            { -0.08473049309397777, -0.04820837781551201, -0.04820837781551189, 0.795480226200906, 0.1928335112620478, 0.1928335112620478 } } };
    // Unstructured piecewise computations
    const double J_c0 = coordinate_dofs[0] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[2] * FE3_C0_D01_Q6[0][0][1];
    const double J_c3 = coordinate_dofs[1] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[5] * FE3_C0_D01_Q6[0][0][1];
    const double J_c1 = coordinate_dofs[0] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[4] * FE3_C0_D01_Q6[0][0][1];
    const double J_c2 = coordinate_dofs[1] * FE3_C0_D01_Q6[0][0][0] + coordinate_dofs[3] * FE3_C0_D01_Q6[0][0][1];
    alignas(32) double sp[4];
    sp[0] = J_c0 * J_c3;
    sp[1] = J_c1 * J_c2;
    sp[2] = sp[0] + -1 * sp[1];
    sp[3] = std::abs(sp[2]);
    alignas(32) double BF0[6] = {};
    for (int iq = 0; iq < 6; ++iq)
    {
        // Quadrature loop body setup (num_points=6)
        // Unstructured varying computations for num_points=6
        double w0 = 0.0;
        for (int ic = 0; ic < 6; ++ic)
            w0 += w[0][ic] * FE6_C0_Q6[0][iq][ic];
        alignas(32) double sv6[1];
        sv6[0] = sp[3] * w0;
        const double fw0 = sv6[0] * weights6[iq];
        for (int i = 0; i < 6; ++i)
            BF0[i] += fw0 * FE6_C0_Q6[0][iq][i];
    }
    std::fill(A, A + 6, 0.0);
    for (int i = 0; i < 6; ++i)
        A[i] += BF0[i];
  }

};


class biharmonic_form_0: public ufc::form
{
public:

  biharmonic_form_0() : ufc::form()
  {
    // Do nothing
  }

  ~biharmonic_form_0() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "3853f3bf55ac789d10b87fc6d01f065e26a3d905b731803a9989550a8272cfdeb91b7c195c876032fcc128b23559f0c482375e86d35b2e4b336a7af0dfb7b968";
  }

  std::size_t rank() const final override
  {
    return 2;
  }

  std::size_t num_coefficients() const final override
  {
    return 1;
  }

  std::size_t original_coefficient_position(std::size_t i) const final override
  {
    if (i >= 1)
    {
        throw std::runtime_error("Invalid original coefficient index.");
    }
    static const std::vector<std::size_t> position = {0};
    return position[i];
  }

  ufc::finite_element * create_coordinate_finite_element() const final override
  {
    return new biharmonic_finite_element_1();
  }

  ufc::dofmap * create_coordinate_dofmap() const final override
  {
    return new biharmonic_dofmap_1();
   }

  ufc::coordinate_mapping * create_coordinate_mapping() const final override
  {
    return new biharmonic_coordinate_mapping_1();
  }

  ufc::finite_element * create_finite_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_finite_element_3();
    case 1:
        return new biharmonic_finite_element_3();
    case 2:
        return new biharmonic_finite_element_2();
    default:
        return nullptr;
    }
  }

  ufc::dofmap * create_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_dofmap_3();
    case 1:
        return new biharmonic_dofmap_3();
    case 2:
        return new biharmonic_dofmap_2();
    default:
        return nullptr;
    }
  }

  std::size_t max_cell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_exterior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_vertex_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_custom_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_cutcell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interface_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_overlap_subdomain_id() const final override
  {
    return 0;
  }

  bool has_cell_integrals() const final override
  {
    return true;
  }

  bool has_exterior_facet_integrals() const final override
  {
    return false;
  }

  bool has_interior_facet_integrals() const final override
  {
    return true;
  }

  bool has_vertex_integrals() const final override
  {
    return false;
  }

  bool has_custom_integrals() const final override
  {
    return false;
  }

  bool has_cutcell_integrals() const final override
  {
    return false;
  }

  bool has_interface_integrals() const final override
  {
    return false;
  }

  bool has_overlap_integrals() const final override
  {
    return false;
  }

  ufc::cell_integral * create_cell_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::exterior_facet_integral * create_exterior_facet_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::interior_facet_integral * create_interior_facet_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::vertex_integral * create_vertex_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::custom_integral * create_custom_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::cutcell_integral * create_cutcell_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::interface_integral * create_interface_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::overlap_integral * create_overlap_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::cell_integral * create_default_cell_integral() const final override
  {
    return new biharmonic_cell_integral_0_otherwise();
  }

  ufc::exterior_facet_integral * create_default_exterior_facet_integral() const final override
  {
    return nullptr;
  }

  ufc::interior_facet_integral * create_default_interior_facet_integral() const final override
  {
    return new biharmonic_interior_facet_integral_0_otherwise();
  }

  ufc::vertex_integral * create_default_vertex_integral() const final override
  {
    return nullptr;
  }

  ufc::custom_integral * create_default_custom_integral() const final override
  {
    return nullptr;
  }

  ufc::cutcell_integral * create_default_cutcell_integral() const final override
  {
    return nullptr;
  }

  ufc::interface_integral * create_default_interface_integral() const final override
  {
    return nullptr;
  }

  ufc::overlap_integral * create_default_overlap_integral() const final override
  {
    return nullptr;
  }

};


class biharmonic_form_1: public ufc::form
{
public:

  biharmonic_form_1() : ufc::form()
  {
    // Do nothing
  }

  ~biharmonic_form_1() override
  {
    // Do nothing
  }

  const char * signature() const final override
  {
    return "c7413aa8164636c90ba4c3ff862fa22c04abd80d9a38b876e39edfad333e17740be01bdb72bc871540db8b3aee5c72682537b9a9c77a38742fcc2146dd4ec2fb";
  }

  std::size_t rank() const final override
  {
    return 1;
  }

  std::size_t num_coefficients() const final override
  {
    return 1;
  }

  std::size_t original_coefficient_position(std::size_t i) const final override
  {
    if (i >= 1)
    {
        throw std::runtime_error("Invalid original coefficient index.");
    }
    static const std::vector<std::size_t> position = {0};
    return position[i];
  }

  ufc::finite_element * create_coordinate_finite_element() const final override
  {
    return new biharmonic_finite_element_1();
  }

  ufc::dofmap * create_coordinate_dofmap() const final override
  {
    return new biharmonic_dofmap_1();
   }

  ufc::coordinate_mapping * create_coordinate_mapping() const final override
  {
    return new biharmonic_coordinate_mapping_1();
  }

  ufc::finite_element * create_finite_element(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_finite_element_3();
    case 1:
        return new biharmonic_finite_element_3();
    default:
        return nullptr;
    }
  }

  ufc::dofmap * create_dofmap(std::size_t i) const final override
  {
    switch (i)
    {
    case 0:
        return new biharmonic_dofmap_3();
    case 1:
        return new biharmonic_dofmap_3();
    default:
        return nullptr;
    }
  }

  std::size_t max_cell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_exterior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interior_facet_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_vertex_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_custom_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_cutcell_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_interface_subdomain_id() const final override
  {
    return 0;
  }

  std::size_t max_overlap_subdomain_id() const final override
  {
    return 0;
  }

  bool has_cell_integrals() const final override
  {
    return true;
  }

  bool has_exterior_facet_integrals() const final override
  {
    return false;
  }

  bool has_interior_facet_integrals() const final override
  {
    return false;
  }

  bool has_vertex_integrals() const final override
  {
    return false;
  }

  bool has_custom_integrals() const final override
  {
    return false;
  }

  bool has_cutcell_integrals() const final override
  {
    return false;
  }

  bool has_interface_integrals() const final override
  {
    return false;
  }

  bool has_overlap_integrals() const final override
  {
    return false;
  }

  ufc::cell_integral * create_cell_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::exterior_facet_integral * create_exterior_facet_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::interior_facet_integral * create_interior_facet_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::vertex_integral * create_vertex_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::custom_integral * create_custom_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::cutcell_integral * create_cutcell_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::interface_integral * create_interface_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::overlap_integral * create_overlap_integral(std::size_t subdomain_id) const final override
  {
    return nullptr;
  }

  ufc::cell_integral * create_default_cell_integral() const final override
  {
    return new biharmonic_cell_integral_1_otherwise();
  }

  ufc::exterior_facet_integral * create_default_exterior_facet_integral() const final override
  {
    return nullptr;
  }

  ufc::interior_facet_integral * create_default_interior_facet_integral() const final override
  {
    return nullptr;
  }

  ufc::vertex_integral * create_default_vertex_integral() const final override
  {
    return nullptr;
  }

  ufc::custom_integral * create_default_custom_integral() const final override
  {
    return nullptr;
  }

  ufc::cutcell_integral * create_default_cutcell_integral() const final override
  {
    return nullptr;
  }

  ufc::interface_integral * create_default_interface_integral() const final override
  {
    return nullptr;
  }

  ufc::overlap_integral * create_default_overlap_integral() const final override
  {
    return nullptr;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/mesh/Mesh.h>
#include <dolfin/mesh/MultiMesh.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/fem/MultiMeshForm.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/MultiMeshFunctionSpace.h>
#include <dolfin/function/GenericFunction.h>
#include <dolfin/function/CoefficientAssigner.h>
#include <dolfin/function/MultiMeshCoefficientAssigner.h>
#include <dolfin/adaptivity/ErrorControl.h>
#include <dolfin/adaptivity/GoalFunctional.h>
#include <dolfin/la/GenericVector.h>

namespace Biharmonic
{

class CoefficientSpace_alpha: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  CoefficientSpace_alpha(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_2>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_2>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  CoefficientSpace_alpha(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_2>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_2>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  CoefficientSpace_f(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  CoefficientSpace_f(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_a_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_a_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_FunctionSpace_1: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_a_FunctionSpace_1(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_a_FunctionSpace_1(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_a_MultiMeshFunctionSpace_0: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_a_MultiMeshFunctionSpace_0(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_a_FunctionSpace_0(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

class Form_a_MultiMeshFunctionSpace_1: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_a_MultiMeshFunctionSpace_1(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_a_FunctionSpace_1(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

typedef CoefficientSpace_alpha Form_a_FunctionSpace_2;

class Form_a: public dolfin::Form
{
public:

  // Constructor
  Form_a(std::shared_ptr<const dolfin::FunctionSpace> V1, std::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(2, 1), alpha(*this, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    _ufc_form = std::make_shared<const biharmonic_form_0>();
  }

  // Constructor
  Form_a(std::shared_ptr<const dolfin::FunctionSpace> V1, std::shared_ptr<const dolfin::FunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> alpha):
    dolfin::Form(2, 1), alpha(*this, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    this->alpha = alpha;

    _ufc_form = std::make_shared<const biharmonic_form_0>();
  }

  // Destructor
  ~Form_a()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "alpha")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "alpha";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_a_FunctionSpace_0 TestSpace;
  typedef Form_a_FunctionSpace_1 TrialSpace;
  typedef Form_a_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_a_MultiMeshFunctionSpace_1 MultiMeshTrialSpace;
  typedef Form_a_FunctionSpace_2 CoefficientSpace_alpha;

  // Coefficients
  dolfin::CoefficientAssigner alpha;
};

class MultiMeshForm_a: public dolfin::MultiMeshForm
{
public:

  // Constructor
  MultiMeshForm_a(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V1, std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0):
    dolfin::MultiMeshForm(V1, V0), alpha(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_a(V1->part(part), V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients

  }

  // Constructor
  MultiMeshForm_a(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V1, std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> alpha):
    dolfin::MultiMeshForm(V1, V0), alpha(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_a(V1->part(part), V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients
    this->alpha = alpha;

  }

  // Destructor
  ~MultiMeshForm_a()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "alpha")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "alpha";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_a_FunctionSpace_0 TestSpace;
  typedef Form_a_FunctionSpace_1 TrialSpace;
  typedef Form_a_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_a_MultiMeshFunctionSpace_1 MultiMeshTrialSpace;
  typedef Form_a_FunctionSpace_2 CoefficientSpace_alpha;

  // Coefficients
  dolfin::MultiMeshCoefficientAssigner alpha;
};

class Form_L_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  // Constructor for standard function space
  Form_L_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh))
  {
    // Do nothing
  }

  // Constructor for constrained function space
  Form_L_FunctionSpace_0(std::shared_ptr<const dolfin::Mesh> mesh, std::shared_ptr<const dolfin::SubDomain> constrained_domain):
    dolfin::FunctionSpace(mesh,
                          std::make_shared<const dolfin::FiniteElement>(std::make_shared<biharmonic_finite_element_3>()),
                          std::make_shared<const dolfin::DofMap>(std::make_shared<biharmonic_dofmap_3>(), *mesh, constrained_domain))
  {
    // Do nothing
  }

};

class Form_L_MultiMeshFunctionSpace_0: public dolfin::MultiMeshFunctionSpace
{
public:

  // Constructor for multimesh function space
  Form_L_MultiMeshFunctionSpace_0(std::shared_ptr<const dolfin::MultiMesh> multimesh): dolfin::MultiMeshFunctionSpace(multimesh)
  {
    // Create and add standard function spaces
    for (std::size_t part = 0; part < multimesh->num_parts(); part++)
    {
      std::shared_ptr<const dolfin::FunctionSpace> V(new Form_L_FunctionSpace_0(multimesh->part(part)));
      add(V);
    }

    // Build multimesh function space
    build();
  }

};

typedef CoefficientSpace_f Form_L_FunctionSpace_1;

class Form_L: public dolfin::Form
{
public:

  // Constructor
  Form_L(std::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    _ufc_form = std::make_shared<const biharmonic_form_1>();
  }

  // Constructor
  Form_L(std::shared_ptr<const dolfin::FunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = f;

    _ufc_form = std::make_shared<const biharmonic_form_1>();
  }

  // Destructor
  ~Form_L()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_L_FunctionSpace_0 TestSpace;
  typedef Form_L_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_L_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::CoefficientAssigner f;
};

class MultiMeshForm_L: public dolfin::MultiMeshForm
{
public:

  // Constructor
  MultiMeshForm_L(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0):
    dolfin::MultiMeshForm(V0), f(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_L(V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients

  }

  // Constructor
  MultiMeshForm_L(std::shared_ptr<const dolfin::MultiMeshFunctionSpace> V0, std::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::MultiMeshForm(V0), f(*this, 0)
  {
    // Create and add standard forms
    std::size_t num_parts = V0->num_parts(); // assume all equal and pick first
    for (std::size_t part = 0; part < num_parts; part++)
    {
      std::shared_ptr<dolfin::Form> a(new Form_L(V0->part(part)));
    add(a);

    }
    // Build multimesh form
    build();

    /// Assign coefficients
    this->f = f;

  }

  // Destructor
  ~MultiMeshForm_L()
  {}

  /// Return the number of the coefficient with this name
  virtual std::size_t coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(std::size_t i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::dolfin_error("generated code for class Form",
                         "access coefficient data",
                         "Invalid coefficient");
    return "unnamed";
  }

  // Typedefs
  typedef Form_L_FunctionSpace_0 TestSpace;
  typedef Form_L_MultiMeshFunctionSpace_0 MultiMeshTestSpace;
  typedef Form_L_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::MultiMeshCoefficientAssigner f;
};

// Class typedefs
typedef Form_a BilinearForm;
typedef MultiMeshForm_a MultiMeshBilinearForm;
typedef Form_a JacobianForm;
typedef MultiMeshForm_a MultiMeshJacobianForm;
typedef Form_L LinearForm;
typedef MultiMeshForm_L MultiMeshLinearForm;
typedef Form_L ResidualForm;
typedef MultiMeshForm_L MultiMeshResidualForm;
typedef Form_a::TestSpace FunctionSpace;
typedef Form_a::MultiMeshTestSpace MultiMeshFunctionSpace;

}

#endif
